<?php
namespace HCM\FPA\Controllers;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Controllers\RelatoriosHcmController;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Util;
use HCM\Util\FPA;
use HCM\Util\Repositories;
use HCM\FPA\Services\FormularioCagedService;

class FormularioCagedController extends Crud {

    protected $dataSourceGetFormularioCaged = 'fpaFormCaged_getFormularioCaged';
    protected $dataSourceSetFormularioCaged = 'fpaFormCaged_setFormularioCaged';

    protected $parameterBagColumns = array('P_NRORG', 'P_CDOPERADOR', 'P_NRTIPOESTRUTURA', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters,
    EntityManager $entityManager,FormularioCagedService $formularioCagedService, FPA $fpa, RelatoriosHcmController $relatoriosHcmController){
        
        $this->parameterBag     = $parameterBag;
        $this->hcmParameters    = $hcmParameters;
        $this->entityManager = $entityManager;
        $this->formularioCagedService = $formularioCagedService;
        $this->relatoriosHcmController = $relatoriosHcmController;
        parent::__construct($dataSourceManager);
    }

    public function getFormularioCaged (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetFormularioCaged);
            $conditions = $filterData->getConditions();
            $conditions[] = [
                'columnName' => 'P_NRTIPOESTRUTURA',
                'value' => $this->hcmParameters->getTipoEstruturaLegalOrg(),
                'operator' => '='
            ];
            $filterDataNew = new FilterCriteria(
                $filterData->getDataSourceName(),
                $conditions,
                $filterData->getPage(),
                $filterData->getPageSize()
            );
            $newFilterData = $this->populateParameterBag($filterDataNew);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $rows = $dataSet->getRows();
            
            foreach($rows as $key => &$row){
                if($row['DSORDEM'] == 'LISTA_VINCULO'){
                    $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIO']);
                    foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                        $row['NRVINCULOM_LIST'][$chave] = intval($value);
                    
                    $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
                }    
                
                if($row['DSORDEM'] == 'LISTA_ESTRUTURA'){
                    $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIO']);
                    foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                        $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                    
                    if($row['DSFIM']){
                        $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIM']);
                        foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                            $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                        
                        $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                
                    }

                }
                
                if($row['DSORDEM'] === "LISTA_NRESTRUTSIND"){
                    if($row['DSFIM']) {
                        $row['LIST_NRESTRUTSIND'] = explode(',',$row['DSFIM']);
                        foreach($row['LIST_NRESTRUTSIND'] as $chave => $value)
                            $row['LIST_NRESTRUTSIND'][$chave] = strval($value);
                            
                        $row['LIST_NMESTRUTSIND'] = explode(",", $row['NMESTRUTSIND']);
                    }
                }
            }
            $response->addDataSet(new DataSet($this->dataSourceGetFormularioCaged, $rows));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setFormularioCaged (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetFormularioCaged;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new'] === true){
                    $row['NRFORMCAGED'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_FORMCAGED', 1, 12, true);
                }
                if(!array_key_exists('DSINICIO', $row)){
                    $row['DSINICIO'] = null;
                }
                if(!array_key_exists('DSFIM', $row)){
                    $row['DSFIM'] = null;
                }
                
                // define os campos de descrição de início e fim com base no critério de seleção passado pela row
                if($row['DSORDEM'] === "LISTA_VINCULO"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                }

                if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if(isset($row['DSFIM']) && count($row['DSFIM']) > 0 && is_array($row['DSFIM'])){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                
                if($row['DSORDEM'] === "LISTA_NRESTRUTSIND") {
                    if(count($row['DSFIM']) > 0 && is_array($row['DSFIM'])){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function gerarCAGED(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                $params = array(
                    "P_COMPETENCIA" => $this->hcmParameters->getDtCompetencia(),
                    "P_ESTRUTURARESP" => $row['NRESTRUTRESPONS'],
                    "P_TIPOHIERARQUIA" => $row['NRHIERARQORG'],
                    "P_ESTRUTURARESP" => $row['NRESTRUTRESPONS']
                );
                
                error_reporting(E_ERROR | E_PARSE);
                $nrFormulario = $row['NRFORMCAGED'];
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $cdOperador = $this->hcmParameters->getCdoperador();
                $mesCompetencia = Util::datadeString($this->hcmParameters->getDtCompetencia(), true);
                $magnetico = $this->entityManager->getRepository(Repositories::FPA_CAGEDMAGNET)->findOneBy(array(
                    "nrorg" => $nrorg, "mescompetencia" => $mesCompetencia, "cdoperinclusao" => $cdOperador, "nrorginclusao" => $nrorg));
                
                if (is_object($magnetico)) {
                    if($row['IDCONTINUA'] == 'S'){
                        $message = $this->gerarMagneticoCaged($nrFormulario, $row);
                    }
                } else{
                    $message = $this->gerarMagneticoCaged($nrFormulario, $row);
                }
                if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                    $response->addMessage($message);
                    if ($message->getType() == DTO\Response\Message::TYPE_MESSAGE) {
                        $this->relatoriosHcmController->setRelatoriosHcmBackend($params,'analitico_caged', $request, $response);
                        $this->relatoriosHcmController->setRelatoriosHcmBackend($params,'analitico_caged_', $request, $response);
                    }
                } else {
                    $row = array();
                    foreach ($message as $mensagem) {
                        $row[] = array("NRMSGCALC" => $mensagem->getNrmsgcalc(), "DSDESCRICAO" => $mensagem->getDescricao(), "IDTIPOMSG" => $mensagem->getIdtipomsg());
                    }
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('gerarMagneticoCaged', $row));
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('CAGED finalizado com advertências. Verifique as mensagens.', DTO\Response\Message::TYPE_WARNING));
                    $this->relatoriosHcmController->setRelatoriosHcmBackend($params,'analitico_caged', $request, $response);
                    $this->relatoriosHcmController->setRelatoriosHcmBackend($params,'analitico_caged_', $request, $response);
                }
            }
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
        }
    }

    public function downloadCaged(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            error_reporting(E_ERROR | E_PARSE);
            $this->formularioCagedService->geraMagnetico($request->getDataSet()->getRows()[0]['NRFORMCAGED']);

        }catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function retrocederCAGED(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->formularioCagedService->retroceder();
            $message = new \Zeedhi\Framework\DTO\Response\Message('Retrocedido com sucesso', DTO\Response\Message::TYPE_MESSAGE);
            $response->addMessage($message);
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

     public function gerarMagneticoCaged($nrFormulario, $row){
        
        set_time_limit(0);

        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $cdOperador = $this->hcmParameters->getCdOperador();
        $resultado = $this->formularioCagedService->percorreCompetenciaMensal($nrFormulario);
        
        $message = null;
        if ($resultado === null) {
            throw new \Exception('Arquivo não foi gerado');
        } elseif ($resultado['ABORT'] === true) {
            $message = 'CAGED com erros. Não finalizado.<br>';
            
            //ver como vai para o front para poder funcionar
            foreach ($resultado['MENSAGENS'] as $mensagem) {
                $message .= strpos($message, $mensagem['MENSAGEM']) ? null : '<br> - ' . $mensagem['MENSAGEM'];
            }
            $message = new \Zeedhi\Framework\DTO\Response\Message($message, DTO\Response\Message::TYPE_ERROR);
        } elseif ($resultado['ERRO'] === false) {
            $message = new \Zeedhi\Framework\DTO\Response\Message('Arquivo gerado com sucesso', DTO\Response\Message::TYPE_MESSAGE);
        } else {
            $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $resultado['CALCULO'], 'dsorigmsg' => 'CAGED', 'nrorg' => $this->hcmParameters->getNrorgTrabalho()));
        }
        return $message;
    }

    public function deleteFormularioCaged (DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = $this->dataSourceSetFormularioCaged;
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}