<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\FPA;

class LayoutAberturaContaController extends Crud {

    protected $dataSourceNameGet = 'getLayoutAbCta';
    protected $parameterBagColumns = ['P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC' ,'P_NRTIPOHIERARQORG', 'P_NRESTRUTSUPERIORM', 'P_DTADMISSAOVINC_INI', 'P_DTADMISSAOVINC_FIM'];

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        parent::__construct($dataSourceManager);
    }

    public function getLayoutAbCta (DTO\Request\Filter $request, DTO\Response $response) { 
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        
        foreach ($filterData->getConditions() as $filter) {
            if($filter['columnName'] == 'NRTIPOHIERARQORG') {
                $filterData->addCondition('P_NRTIPOHIERARQORG', '=', $filter['value']);
            }
            if($filter['columnName'] == 'NRESTRUTSUPERIORM') {
                $filterData->addCondition('P_NRESTRUTSUPERIORM', 'IN', implode($filter['value'], ','));
            }
        }

        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function gerarCSV(DTO\Request\DataSet $request, DTO\Response $response){
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        
        $dataSet = $request->getDataSet()->getRows()[0];

        try{
            $arquivo = $this->writeCSV($dataSet);

            if($arquivo){
                $row = ['NMARQUIVO' => $arquivo];

                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', [$row]));
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Arquivo CSV finalizado com sucesso!', DTO\Response\Message::TYPE_MESSAGE));
            }
            else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Erro na geração do CSV. Tente novamente', DTO\Response\Message::TYPE_MESSAGE));
            }
        } catch(\Exception $e){
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage()));
        }
    }

    private function writeCSV($dataSet) {
        $dataIgnore = ['__is_new','__isSelected','__notSelectable',
                       'NRPESSOA','NRORG','NRVINCULOM','NRTIPOVINCULOM','NRESTRUTLEGAL','NMESTRUTLEGAL',
                       'NRESTRUTGEREN','NMESTRUTGEREN','NRESTRUTSIND','NMESTRUTSIND','NRTIPOHIERARQORG',
                       'NRESTRUTSUPERIORM','NMESTRUTSUPERIORM'];

        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $filePath = FPA::getTempDirectory().DIRECTORY_SEPARATOR;
        $fileName = 'file.csv';
        $file = $filePath.$fileName;

        $fp = fopen($file, "w");

        foreach ($dataSet as $linha) {
            foreach($linha as $index => &$value) {
                if(in_array($index, $dataIgnore)) {
                    unset($linha[$index]);
                }
                if($nrorg == 1410) { //Tratamento exclusivo para o cliente Masan(Agile).
                    if($index == 'CDINTEGRACAO') //25(Y) COD OCUPACAO
                        $value = 130;
                    if($index == 'NATOCUPACAO')  //27(AA) NATUREZA OCUPAÇÃO
                        $value = 1;
                }
            }
            fputcsv($fp, $linha, ';');
        }

        fclose($fp);

        if($fp)
            return $file;
        else
            return false;
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {
        $conditions = [];
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );

        return $newFilterCriteria;
    }
}