<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Notification;
use HCM\Util\HCMParameters;
use HCM\Util\Util;
use HCM\Util\DateUtil;
use Zeedhi\Framework\DataSource\DataSet;

class MonitCalculoController extends Crud {

    protected $regraService;
    protected $dataSourceNameGet = 'fpa_calculoconsumer';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters ,\HCM\FPA\Services\MonitCalcService $monitCalcService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->monitCalcService = $monitCalcService;
        parent::__construct($dataSourceManager);
    }

    public function getMonitCalc(DTO\Request\Filter $request, DTO\Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $dataSet = $this->monitCalcService->getMonitCalc($filters);
        $response->addDataSet(new Dataset("getMonitCalc",$dataSet));
    }
    
    public function getMensagensMonit(DTO\Request\Filter $request, DTO\Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $dataSet = $this->monitCalcService->getMensagensMonit($filters);
        $response->addDataSet(new Dataset("getMensagensMonit",$dataSet));
    }
    
    public function deleteMonitCalc(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        } 
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 