<?php

namespace HCM\FPA\Controllers;

use HCM\FPA\Services\ResumoContabilService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class ResumoContabilController extends Crud {

    protected $dataSourceName = 'fpa_resumoctbl';
    protected $dataSourceNameGet = 'getResumoctbl';

    protected $parameterBagColumns = array('P_NRORG','P_DTMESCOMPETENC', 'P_NRTIPOESTRUTLEGAL', 'P_NRTIPOESTRUTGEREN', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, ResumoContabilService $resumoContabilService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->resumoContabilService = $resumoContabilService;
        parent::__construct($dataSourceManager);
    }

    public function getResumoctbl(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $conditions = $filterData -> getConditions();
        foreach($conditions as &$condition){
            if($condition['columnName'] == 'CDCONTCTBL') {
                $condition['operator'] = 'IN';
            }
            if($condition['columnName'] == 'CDHISTPADR') {
                $condition['operator'] = 'IN';
            }
        }
        $filterData = new FilterCriteria(
            $filterData->getDataSourceName(),
            $conditions,
            $filterData->getPage(),
            $filterData->getPageSize()
        );
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $filterData->addCondition('P_NRTIPOESTRUTLEGAL', $this->hcmParameters->getTipoEstruturaLegalOrg());
        $filterData->addCondition('P_NRTIPOESTRUTGEREN', $this->hcmParameters->getTipoEstruturaGerencialOrg());
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setResumoctbl(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row['NRLANCAMENTO'] = isset($row['NRLANCAMENTO']) ? $row['NRLANCAMENTO'] : null;
                $row['NRTIPOMOVIMENT'] = isset($row['NRTIPOMOVIMENT']) ? $row['NRTIPOMOVIMENT'] : null;
                $row['IDTIPOLANC'] = isset($row['IDTIPOLANC']) ? $row['IDTIPOLANC'] : null;
                $row['CDCONTCTBL'] = isset($row['CDCONTCTBL']) ? $row['CDCONTCTBL'] : null;
                $row['CDHISTPADR'] = isset($row['CDHISTPADR']) ? $row['CDHISTPADR'] : null;

                if (array_key_exists('NRRESUMOCTBL', $row)) {
                    if ($row['NRRESUMOCTBL'] === '') {
                        $row['NRRESUMOCTBL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RESUMOCTBL', 1, 12, true);
                    }
                } else {
                    $row['NRRESUMOCTBL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RESUMOCTBL', 1, 12, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteResumoctbl(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    public function gerarLotePadrao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dtocorrencia = $this->hcmParameters->getDtCompetencia();
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $cdoperador = $this->hcmParameters->getCdoperador();
                $nrtipomoviment = $row['NRTIPOMOVIMENT'];
                $reversao = $row['REVERTELOTE'];
                $unificararq = $row['UNIFICARARQ'];
                $listaestrutsup = isset($row['NRESTRUTSUP']) ? implode(',', $row['NRESTRUTSUP']) : null;
                $dtcontabil = isset($row['DTCONTABIL']) ? $row['DTCONTABIL'] : null;
                
                $this->resumoContabilService->geraArquivoLotePadrao($dtocorrencia, $nrorg, $cdoperador, $response, $dtcontabil, $nrtipomoviment, $listaestrutsup, $reversao, $unificararq);
                
                $data = array();
                foreach ($response->getDataSets() as $dataset) {
                    $data[]=$dataset->getRows();
                }
                if(!!$data) {
                    $response->setDataSets(array(new \Zeedhi\Framework\DataSource\DataSet('arquivos', $data)));
                } else {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Nenhum resumo encontrado para os parametros utilizados.", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }

    public function geraArquivoProvisaoFerias(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dtCompetencia = $this->hcmParameters->getDtCompetencia();
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $cdoperador = $this->hcmParameters->getCdoperador();
                $nrEstrutLegal = isset($row['NRESTRUTLEGAL']) ? $row['NRESTRUTLEGAL'] : null;
                $dtContabil = isset($row['DTCONTABIL']) ? $row['DTCONTABIL'] : $dtCompetencia;
                
                $resumo = $this->resumoContabilService->geraArquivoProvisaoFerias($nrorg, $nrEstrutLegal, $dtContabil);
                
                if(!empty($resumo)) {
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', array($resumo)));
                } else {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Nenhum resumo encontrado para os parametros utilizados.", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function geraArquivoProvisao13(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dtCompetencia = $this->hcmParameters->getDtCompetencia();
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $cdoperador = $this->hcmParameters->getCdoperador();
                $nrEstrutLegal = isset($row['NRESTRUTLEGAL']) ? $row['NRESTRUTLEGAL'] : null;
                $dtContabil = isset($row['DTCONTABIL']) ? $row['DTCONTABIL'] : $dtCompetencia;
                
                $resumo = $this->resumoContabilService->geraArquivoProvisao13($nrorg, $nrEstrutLegal, $dtContabil);
                
                if(!empty($resumo)) {
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', array($resumo)));
                } else {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Nenhum resumo encontrado para os parametros utilizados.", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function geraArquivoFolha13Ferias(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dtCompetencia = $this->hcmParameters->getDtCompetencia();
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $cdoperador = $this->hcmParameters->getCdoperador();
                $nrEstrutLegal = isset($row['NRESTRUTLEGAL']) ? $row['NRESTRUTLEGAL'] : null;
                $dtContabil = isset($row['DTCONTABIL']) ? $row['DTCONTABIL'] : $dtCompetencia;
                
                $resumo = $this->resumoContabilService->geraArquivoFolha13Ferias($nrorg, $nrEstrutLegal, $dtContabil);
                
                if(!empty($resumo)) {
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', array($resumo)));
                } else {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Nenhum resumo encontrado para os parametros utilizados.", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function geraArquivoFolha(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dtCompetencia = $this->hcmParameters->getDtCompetencia();
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $cdoperador = $this->hcmParameters->getCdoperador();
                $nrEstrutLegal = isset($row['NRESTRUTLEGAL']) ? $row['NRESTRUTLEGAL'] : null;
                $dtContabil = isset($row['DTCONTABIL']) ? $row['DTCONTABIL'] : $dtCompetencia;
                
                $resumo = $this->resumoContabilService->geraArquivoFolha($nrorg, $nrEstrutLegal, $dtContabil);
                
                if(!empty($resumo)) {
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', array($resumo)));
                } else {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Nenhum resumo encontrado para os parametros utilizados.", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
} 