<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\Remote\cURLRequest;
use Zeedhi\Framework\Remote\HttpInterface;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Geral\Services\Queue\HCMProducerService;
use HCM\FPA\Repository\calculoFolhaRepository;
use HCM\Geral\Services\HcmServices;

class CalculoFolhaService {
    
    protected $entityManager;
    protected $hcmParameters;
    protected $vinculoServices;
    protected $hcmProducerService;
    protected $hcmServices;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, \HCM\Geral\Services\VinculoServices $vinculoServices, HCMProducerService $hcmProducerService,
    calculoFolhaRepository $calculoFolhaRepository, HcmServices $hcmServices) {
        $this->entityManager      = $entityManager;
        $this->hcmParameters      = $hcmParameters;
        $this->vinculoServices    = $vinculoServices;
        $this->hcmProducerService = $hcmProducerService;
        $this->calculoFolhaRepository = $calculoFolhaRepository;
        $this->hcmServices   = $hcmServices;
    }
    
	public function calculo($dataset) {
		$nrorg          = $this->hcmParameters->getNrorgTrabalho();
		$nrorgPadrao    = $this->hcmParameters->getNrorgPadrao();
		$cdoperador     = $this->hcmParameters->getCdoperador();
		$mesCompetencia = $this->hcmParameters->getDtCompetencia();
		$dtUltDiaMes    = $this->hcmParameters->getUltimoDiaMes();
		$nrTipoCalculo  = $dataset['TP_CALCULO'];
        
		if ($nrTipoCalculo === \Folha\Entities\FpaTipocalculo::CALC_RESCISAO ||
            $nrTipoCalculo === \Folha\Entities\FpaTipocalculo::CALC_RESCISAO_COMPLEMENTAR)  {
			
            if ($nrTipoCalculo === \Folha\Entities\FpaTipocalculo::CALC_RESCISAO) {
                $vinculos = $this->vinculoServices->retornaRescindidosGeral(
                    $mesCompetencia, $dataset['DSINICIO'], $dataset['DSFIM'], $mesCompetencia, $dtUltDiaMes, $dataset['DSORDEM']
                );
            };

            if ($nrTipoCalculo === \Folha\Entities\FpaTipocalculo::CALC_RESCISAO_COMPLEMENTAR) {
            	$vinculos = $this->vinculoServices->retornaRescindidosGeral(
                    $mesCompetencia, $dataset['DSINICIO'], $dataset['DSFIM'], null, null, $dataset['DSORDEM']
                );
            };

        } else {
            if ($nrTipoCalculo === \Folha\Entities\FpaTipocalculo::CALC_ADIANT_PLR || $nrTipoCalculo === \Folha\Entities\FpaTipocalculo::CALC_PLR) {
                // @todo verificar implementação
            } else {
                $vinculos = $this->vinculoServices->retornaVinculosCompetencia(
                    $nrorg, $mesCompetencia, false, $dataset['DSINICIO'], $dataset['DSFIM'], $dataset['DSORDEM'], false
                );
            }
        	
        }
        
        $nrvinculos = array();
        foreach ($vinculos as $vinculo) {
            $nrvinculos[$vinculo['NRVINCULOM']] = $vinculo['NMVINCULOM'];
        }

        // cria registro no calculoconsumer
        $nrcalculoconsumer = $this->insereCalculoConsumer($nrorg, count($nrvinculos), $dataset['DSORDEM'], $dataset['DSINICIO'], $dataset['DSORDEM']);

        $calculoParams = array(
            "NRCALCULOCONSUMER"  => $nrcalculoconsumer,
	        "CDOPERADOR"         => $cdoperador,
            "DSEMAILOPER"        => $dataset["DSEMAILOPER"],
            "DTMESCOMPETENCIA"   => $this->hcmParameters->getDtCompetencia(),
            "NRORG"              => $nrorg,
            "NRFORMULARIO"       => $dataset["NRFORMCALCULO"],
	        "TIPO_PROCESSAMENTO" => $dataset["TP_PROCESSAMENTO"],
	        "NRTIPOCALCULO"      => $dataset["TP_CALCULO"],
	        "VINCULOS"           => $nrvinculos
	    );

        $this->hcmProducerService->sendCalculo($calculoParams);
        $this->inicializaConsumidorJenkins();

    } 
    
    public function enviaCalculoParaHcmServices($route, $params, $nrOrg, $post){
        try {
            $this->hcmServices->sendAsync($route, $params, $nrOrg, $post);
        } catch (\Exception $e) {
            throw $e; 
        }
    }

    public function insereCalculoConsumer($nrorg, $nrtotvinculos, $dsordem, $dsinicio, $dsfim) {
        $calculoConsumer   = new \Teknisa\Model\Gestaopessoas\Entities\FpaCalculoconsumer();
        $nrcalculoconsumer = \HCM\Util\NovoCodigo::geraCodigo($nrorg, 'FPA_CALCULOCONSUMER', 1, 20, true);

        $this->entityManager->beginTransaction();

        try {
            $calculoConsumer->setNrcalculoconsumer($nrcalculoconsumer);
            $calculoConsumer->setNrorg($nrorg);
            $calculoConsumer->setNrtotvinculos($nrtotvinculos);
            $calculoConsumer->setIdsituacao(\Teknisa\Model\Gestaopessoas\Entities\FpaCalculoconsumer::IDSITUACAO_PENDENTE);
            $calculoConsumer->setDtinclusao(\HCM\Util\DateUtil::getDataAtual());
            $calculoConsumer->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $calculoConsumer->setNrorginclusao($nrorg);
            $calculoConsumer->setDsOrdem($dsordem);
            if(is_array($dsinicio)){
                $dsinicio = implode(',', $dsinicio);
                $calculoConsumer->setDsInicio($dsinicio);
            } else {
                $calculoConsumer->setDsInicio($dsinicio);
            }
            
            if(is_array($dsfim)){
                $dsfim = implode(',', $dsfim);
                $calculoConsumer->setDsFim($dsfim);
            } else {
                $calculoConsumer->setDsFim($dsfim);
            }

            $this->entityManager->persist($calculoConsumer);
            $this->entityManager->flush();
            $this->entityManager->commit();
        } catch(\Exception $e) {
            $this->entityManager->rollback();
            throw new \Exception($e->getMessage());
        }

        return $nrcalculoconsumer;
    }

    private function inicializaConsumidorJenkins() {
        $url = $this->hcmParameters->getUrlJenkinsCalculoConsumer();
        $cURLRequest = new cURLRequest($url);
        $cURLRequest->request('');

        $requestInfo = $cURLRequest->getRequestInfo();    
        if ($requestInfo['http_code'] !== 201) {
            throw new \Exception(Messages::HCM01000_FPA_CALCULOCONSUMER_JENKINS);
        }  
    }
    
    public function carregaListaNrCalculoFolhaTodasOcorrencias(
        $nrOrg, $tipoMovimento, $dtOcorrencia, $nrTpModalidadeCalc
    ){
        $calculosFolha = $this->calculoFolhaRepository->retornaCalculoFolhaTodasOcorrencias(
            $nrOrg, $tipoMovimento, $nrTpModalidadeCalc, $dtOcorrencia, $dtOcorrencia
        );

        $listaNrCalculo = array();
        
        if(count( $calculosFolha) > 0 ){
            foreach ($calculosFolha as $calculo){
                $listaNrCalculo[] = $calculo->getNrcalculofolha();
            }
        }
        
        return $listaNrCalculo;
    }
    
    public function retornaCalculoFolha(
        $nrorg, $tipoMovimento, $dtOcorrencia, $nrTpModalidadeCalc, $nrOcorrenciaCalc
    ){
        $calculosFolha = $this->calculoFolhaRepository->retornaCalculoFolha(
            $nrorg, $tipoMovimento, $nrTpModalidadeCalc, $nrOcorrenciaCalc, $dtOcorrencia
        );
       
        return $calculosFolha;
    }
}