<?php

namespace HCM\Geral\Controllers;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\Controller\Crud;
use HCM\Util\DateUtil;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use HCM\Geral\Services\AprovacaoAdmissaoService;
use HCM\Geral\Services\ESocialService;
use HCM\Util\Util;

class AprovacaoAdmissaoController extends Crud {

    protected $parameterBagColumns = array();
    protected $dataSourceName = '';

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, AprovacaoAdmissaoService $aprovacaoAdmissaoService, ESocialService $eSocialService, EntityManager $entityManager){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->aprovacaoAdmissaoService = $aprovacaoAdmissaoService;
        $this->eSocialService = $eSocialService; 
        $this->entityManager = $entityManager;
        parent::__construct($dataSourceManager);
    }
    
    public function getVinculosNaoAprovados(DTO\Request\Filter $request, DTO\Response $response) {
        try{
            set_time_limit(0); 
           $filters = $request->getFilterCriteria()->getConditions();
           $parameters = array();
           foreach ($filters as $position => $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                    }
                }
            }
            Util::getValueInFilterDataSourceByName($filters, array('P_NRORG', 'P_DTMESCOMPETENC', 'P_NRORG_PADRAO'), true);
            $conditions = Util::createQueryFiltered($filters);
            $result = $this->aprovacaoAdmissaoService->getVinculosNaoAprovados($parameters, $conditions);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getVinculosNaoAprovados', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getPessoasNaoAprovadas(DTO\Request\Filter $request, DTO\Response $response) {
        try{
           $filters = $request->getFilterCriteria()->getConditions();
           $parameters = array();
           foreach ($filters as $position => $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                    }
                }
            }
            Util::getValueInFilterDataSourceByName($filters, array('P_NRORG'), true);
            $conditions = Util::createQueryFiltered($filters);
            $result = $this->aprovacaoAdmissaoService->getPessoasNaoAprovadas($parameters, $conditions);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getPessoasNaoAprovadas', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function aprovaAdmissao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $rows = $request->getDataSet()->getRows()[0];
            $retornoEsocial = array();
            $responseClone = clone $response;
            foreach ($rows  as $row) {
                
                $row['NRESTRUTURAM'] = $row['NRESTRUTLEGAL'];
                if($row['NRTIPOVINCULOM'] == 1){
                    $row['NRCHAVE'] = $row['NRVINCULOM'];
                    $row['NMEVENTO'] = 'S-2200';
                }else{
                    $row['NRCHAVE'] = $row['NRVINCULOM'];
                    $row['NMEVENTO'] = 'S-2300';
                }
                
                $this->aprovacaoAdmissaoService->aprovaAdmissao($row);
                
                if(array_key_exists('IDENVIAESOCIAL', $rows[0]) && $rows[0]['IDENVIAESOCIAL'] == 'S'){
                    $row['DTMESCOMPETENC'] = $this->hcmParameters->getDtCompetencia();
                    $this->eSocialService->envioEsocialDescentralizado($row, $responseClone);
                        
                }
            }

            foreach($responseClone->getDataSets() as $dataSet){
                foreach($dataSet->getRows() as &$dataSet) { 
                    $retornoEsocial[] = $dataSet;
                }
                
            } 

            if(!empty($retornoEsocial)) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', $retornoEsocial));
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('aprova', array(true)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function aprovaAdmissaoPessoa(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $rows = $request->getDataSet()->getRows();
            $rows = $rows[0];
            foreach($rows as $row) {
                $res = $this->aprovacaoAdmissaoService->aprovaAdmissaoPessoa($row);
                $response->addMessage( new Response\Message($res['msg'], $res['type']) );
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setAlteracoesAprova(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $msg = null;
            foreach ($request->getDataSet()->getRows()  as $row) {
                $msg = $this->aprovacaoAdmissaoService->setAlteracoesAprova($row);
            }
            
            if(!empty($msg)){
                 $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($msg, DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('aprova', array(true)));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
}