<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\Messages;
use HCM\Util\Util;
use HCM\Util\DateUtil;
use HCM\Util\FPA;

class ValidacaoSistemaController extends Crud {

    protected $ValidacaoSistemaService;
    protected $parameterBagColumns = array();
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\Geral\Services\ValidacaoSistemaService $ValidacaoSistemaService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->validacaoSistemaService = $ValidacaoSistemaService;
        parent::__construct($dataSourceManager);
    }
    
    public function geraValidacao (DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataSet = $dataset[0];
            
            if(count($dataSet['LIST_NRESTRUTLEGAL']) < 1 && count($dataSet['LIST_NRESTRUTGEREN']) < 1 && count($dataSet['NRVINCULOM_LIST']) < 1){
                //$response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Favor informar um filtro para os vinculos a serem contemplados na rotina', DTO\Response\Message::TYPE_MESSAGE));
                throw new \Exception('Favor informar um filtro para os vinculos a serem contemplados na rotina');
            }
            
            $this->validacaoSistemaService->geraValidacao($dataSet);    
            
            $message = new \Zeedhi\Framework\DTO\Response\Message('Validação finalizada com sucesso', DTO\Response\Message::TYPE_MESSAGE);
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            }
        
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    public function getMensagensValidacao(DTO\Request\Filter $request, DTO\Response $response)
    {
        try{
            
            $params = array();
            $params['NRCALCULOFOLHA'] = FPA::determinaNrCalculoFolha(1);
            $params['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
            $params['DTMESCOMPETENC'] = $this->hcmParameters->getDtCompetencia();
            // $params['FIMCOMPETENCIA'] = DateUtil::truncateString(DateUtil::getUltimoDiaDoMes(substr($this->hcmParameters->getDtCompetencia(), 0, 10)));
            $filters = $request->getFilterCriteria()->getConditions();
            
            foreach ($filters as $filter) {
                if($filter['columnName'] === 'LIST_NRESTRUTLEGAL'){
                    $params['NRESTRUTLEGAL'] = $filter['value'];
                } else if($filter['columnName'] === 'LIST_NRESTRUTGEREN'){
                    $params['NRESTRUTGEREN'] = $filter['value'];
                } else if($filter['columnName'] === 'NRTIPOVINCULOM'){
                    $params['NRTIPOVINCULOM'] = $filter['value'];
                } else if($filter['columnName'] === 'IDTIPOMSG'){
                    $params['IDTIPOMSG'] = $filter['value'];
                } else if($filter['columnName'] === 'NRVINCULOM_LIST'){
                    $params['NRVINCULOM'] = $filter['value'];
                }  else if($filter['columnName'] === 'DSORIGMSG'){
                    $params['DSORIGMSG'] = $filter['value'];
                } 
            }
            
            Util::getValueInFilterDataSourceByName($filters, array('IDTIPOMSG', 'LIST_NRESTRUTGEREN', 'LIST_NRESTRUTLEGAL', 'NRTIPOVINCULOM', 'NRVINCULOM_LIST', 'DSORIGMSG'), true);
            $conditions = Util::createQueryFiltered($filters);
            
            $result = $this->validacaoSistemaService->retornaValidacaoSistema($params, $conditions);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getMensagensValidacao', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 