<?php
 
namespace HCM\Geral\Logic;

use HCM\Geral\Logic\ImportacaoPlanilhaVinculoPessoaLogic;
use HCM\Geral\Repository\ImportacaoPlanilhaVinculoRepository;
use Teknisa\Model\Gestaopessoas\Entities\GpeTpmovtransfm;

use Doctrine\ORM\Configuration;
use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\Util;

class ImportacaoPlanilhaVinculoLogic {

    /** @var \Doctrine\ORM\EntityManager */
    protected $entityManager;
    /** @var \HCM\Util\HCMParameters */
    protected $hcmParameters;
    /** @var \Doctrine\DBAL\Connection */
    protected $connection;
    /** @var \Doctrine\ORM\Configuration */
    protected $config;
    /** @var \HCM\Geral\Logic\ImportacaoPlanilhaVinculoPessoaLogic; */
    protected $logicpessoa;

    private $arrDados;
    private $nrorg;
    private $nrorgpadrao;
    private $gpe_Vinculom;
    private $gpe_Vinculoh;
    private $gpeMovimentacao;
    private $gpeAltesitufunc;
    private $gpe_Altesalario;
    private $gpe_alteOcupacao;
    private $pto_ocorrenciadmissao;
    private $pto_progocorrencia;
    private $pto_bancohorasadmi;
    private $pto_altebancohoras;
    private $gpe_alteEscala;
    private $relacionaparc;

    private $parametrosAdmissao;
    
    private $importacaoPlanilhaVinculoRepository;

    public function __construct(EntityManager $entityManager, Connection $connection, Configuration $config, HCMParameters $hcmParameters, Array $arrayDados, ImportacaoPlanilhaVinculoPessoaLogic $logicPessoa) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->connection    = $connection;
        $this->config        = $config;
        $this->arrDados      = $arrayDados;
        $this->nrorg         = $this->hcmParameters->getNrorgTrabalho();
        $this->nrorgpadrao   = $this->hcmParameters->getNrorgPadrao();

        $this->gpe_Vinculom  = Repositories::GPE_VINCULOM;
        $this->gpe_Vinculoh  = Repositories::GPE_VINCULOH;
        $this->gpeMovimentacao = Repositories::GPE_MOVIMENTACAO;
        $this->gpeAltesitufunc = Repositories::GPE_ALTESITUFUNC;
        $this->gpe_Altesalario = Repositories::GPE_ALTESALARIO;
        $this->gpe_alteOcupacao = Repositories::GPE_ALTEOCUPACAO;
        $this->pto_ocorrenciadmissao = Repositories::PTO_OCORRENCIADMISSAO;
        $this->pto_progocorrencia = Repositories::PTO_PROGOCORRENCIA;
        $this->pto_bancohorasadmi = Repositories::PTO_BANCOHORASADMI;
        $this->pto_altebancohoras = Repositories::PTO_ALTEBANCOHORAS;
        $this->gpe_alteEscala = Repositories::GPE_ALTEESCALA;
        $this->relacionaparc = Repositories::RELACIONAPARC;

        $this->parametrosAdmissao = [];
        
        $this->importacaoPlanilhaVinculoRepository = new ImportacaoPlanilhaVinculoRepository($this->connection, $this->entityManager, $this->hcmParameters);
        $this->logicpessoa = $logicPessoa;
    }

    public function criaDados(&$arrayMsgError) {
        try{
            list($dtadmissaovinc,    $idmultvinc,           $nrpessoa,            $newdtmescompetenc, $nmvinculom,
                 $nrestrutlegal,     $nrvinculom,           $nrtipovinculom,      $nrtpadmissao,      $nrvinculoempreg,
                 $nrdiascontratoexp, $nrdiascontratoexppro, $dtexamemedico,       $nrsitufuncm,       $nrocupacaom,
                 $nrescalatrabm,     $idcontribuisind,      $dtbaseferias,        $idremuneracao,     $idtppagamento,
                 $nrestrutgeren,     $nrtpmodalidsal,       $cdcontribindividual, $nrtpmovtransfm,    $dtfimcontrdetermin,
                 $nrestrutsind,      $vrsalario,            $nmpessoapai,         $nmpessoamae,       $nrparcnegocio,
                 $operadorFoiCriado, $dtopcaofgts,          $dtaposentafgts,      $dtprimadmissao)                         = $this->setaDados();

            $arrayVinculo = [
                'DTADMISSAOVINC'       => $dtadmissaovinc,
                'IDMULTVINC'           => $idmultvinc,
                'NRPESSOA'             => $nrpessoa,
                'NEWDTMESCOMPETENC'    => $newdtmescompetenc,
                'NMVINCULOM'           => strtoupper(Util::removeCaracteresEspeciais($nmvinculom)),
                'NRVINCULOM'           => $nrvinculom,
                'NRTIPOVINCULOM'       => $nrtipovinculom,
                'NRTPADMISSAO'         => $nrtpadmissao,
                'NRVINCULOEMPREG'      => $nrvinculoempreg,
                'NRDIASCONTRATOEXP'    => $nrdiascontratoexp,
                'NRDIASCONTRATOEXPPRO' => $nrdiascontratoexppro,
                'DTEXAMEMEDICO'        => $dtexamemedico ?? $dtadmissaovinc,
                'IDCONTRIBUISIND'      => $idcontribuisind,
                'DTBASEFERIAS'         => $dtbaseferias ?? $dtadmissaovinc,
                'IDREMUNERACAO'        => $idremuneracao,
                'IDTPPAGAMENTO'        => $idtppagamento,
                'CDCONTRIBINDIVIDUAL'  => $cdcontribindividual,
                'NRTPMOVTRANSFM'       => $nrtpmovtransfm,
                'DTFIMCONTRDETERMIN'   => $this->calculaDtfimcontrdetermin($dtfimcontrdetermin, $nrdiascontratoexp, $nrdiascontratoexppro, $dtadmissaovinc),
                'DSLIVROADMISSAO'      => null,
                'DTENTREGAEPI'         => $dtadmissaovinc,
                'DTOPCAOFGTS'          => $dtopcaofgts ?? $dtadmissaovinc,
                'DTAPOSENTAFGTS'       => $dtaposentafgts,
                'DTPRIMADMISSAO'       => $dtprimadmissao ?? $dtadmissaovinc,
                'DTVENCCONTREXPER'     => $nrdiascontratoexp ? DateUtil::subtraiIntervalo((DateUtil::adicionaIntervalo($dtadmissaovinc, $nrdiascontratoexp)), 1) : null,
                'NRCLAUSASSECURAT'     => null,
                'IDPAGPROPFER13'       => null,
                'NRMOTIVOADMISSAO'     => null,
                'NRVINCULOMSUBST'      => null,
                'IDPARTARTCLT'         => 'N'
            ];

            $arrayMovimentacao = [
                'NRESTRUTLEGAL'  => $nrestrutlegal,
                'NRESTRUTGEREN'  => $nrestrutgeren,
                'CDMATRICULAESOCIAL'      => null,
                'NRESTRUTSIND'            => $nrestrutsind,
                'NRTIPOTRANSFER_LEGAL'    => 1,
                'NRTIPOTRANSFER_GEREN'    => 2,
                'NRTIPOTRANSFER_SINDICAL' => 1,
                'DTINIMOVIMENT' => $dtadmissaovinc,
                'NRTPMOVTRANSF' => $nrtpadmissao == 1 ? 1 : 2
            ];

            $arrayParametros = [
                'NRSITUFUNCM'         => $nrsitufuncm,
                'NROCUPACAOM'         => $nrocupacaom,
                'NRESCALATRABM'       => $nrescalatrabm,
                'NRTPMODALIDSAL'      => $nrtpmodalidsal,
                'DTINISITUFUNC'       => $dtadmissaovinc,
                'DTALTESALARIO'       => $dtadmissaovinc,
                'NROCORRENCIA'        => 1,
                'NRTIPOALTERA'        => 1,
                'QTHRLIMITESEPHEXTRA' => null,
                'VRSALARIO'           => $vrsalario,
                'CDTIPOSALARIO'       => 'MENSAL',
                'VRDIFERENCAREAL'     => null,
                'VRPERCREAL'          => null,
                'VRPERCALTERA'        => null,
                'VRSALARIOHR'         => null,
                'DTINIOCUPACAO'       => $dtadmissaovinc,
                'DTFIMOCUPACAO'       => null,
                'NRTIPOOCUPACAO'      => 1,
                'NRBANCOHORAS'        => null,
                'DTINIESCALA'         => $dtadmissaovinc,
                'DTFIMESCALA'         => null,
                'NRTURNO'             => null
            ];

            $arrayDependente = [];

            if($nmpessoamae) {
                $arrayDependente[] = [
                    'NMPESSOA'        => $nmpessoamae,
                    'NRTIPORELACIONA' => 1,
                    'NMTIPORELACIONA' => 'Mãe',
                    'IDSEXOPESSOA'    => 'F'
                ];
            }

            if($nmpessoapai) {
                $arrayDependente[] = [
                    'NMPESSOA'        => $nmpessoapai,
                    'NRTIPORELACIONA' => 2,
                    'NMTIPORELACIONA' => 'Pai',
                    'IDSEXOPESSOA'    => 'M'
                ];
            }

            //INSERE VINCULO
            $dataAtual = date('d/m/Y');
            $dataAdmissao = $dtadmissaovinc;
            
            $anoAtual = $this->anoData($dataAtual);
            $anoAdmissao = $this->anoData($dataAdmissao);
            $anoCompetencia = $this->anoData($this->hcmParameters->getDtCompetencia());

            if($arrayVinculo['IDMULTVINC'] == 'UNICO'){
                $vinculoExistente = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculoAtivoPorPessoa($this->nrorg, $nrpessoa, $newdtmescompetenc);
                if($vinculoExistente){
                    throw new \Exception("Já existe outro vínculo cadastrado para a pessoa. O campo 'Múltiplos Vínculos' não pode ser 'UNICO'.");
                }
            }

            $vinculo = $this->verificaIndex($arrayVinculo);
            $movimentacao = $this->verificaIndex($arrayMovimentacao);
            $parametros = $this->verificaIndex($arrayParametros);

            $esoparametro = $this->entityManager->getRepository(Repositories::ESO_PARAMETRO)->findOneBy(['nrorg' => $this->nrorg, 'nrestruturam' => $movimentacao['NRESTRUTLEGAL']]);

            //Verifica se a organização utiliza aprovação de admissão (ID_APROVACAO_ADMISSAO = 1) para setar histórico do vínculo
            $idAprovAdmissao = ($esoparametro ? $esoparametro->getIdaprovadmissao() : 'N');
            if ($anoAtual > $anoAdmissao) {
                $competencia = $newdtmescompetenc;
                $idAprovAdmissao = 'N';
            } else if ($idAprovAdmissao == 'S' || isset($solicitacao['NRTIPOSOLICITACAO'])) {
                $competencia =  Util::datadeString("01/01/3000",true);
            } else {
                $competencia = $newdtmescompetenc;
            }

            //Verifica se utiliza geração de matrícula do esocial
            //Só gera matrícula se não utiliza aprovação de admissão, pois assim gera a matrícula na aprovação
            if ($esoparametro && $esoparametro->getIdaprovadmissao() != 'S' && $esoparametro->getIdutilizamatric() == 'S' && $movimentacao['CDMATRICULAESOCIAL'] == null) {
                $matriculaEsocial = $this->entityManager->getRepository(Repositories::ESO_PARAMETRO)->retonaUltimaMatriculaEsoIncrementada($this->nrorg, $movimentacao['NRESTRUTLEGAL'], $newdtmescompetenc->format('d/m/Y'))['NOVO'];
                $movimentacao['CDMATRICULAESOCIAL'] = $matriculaEsocial;
            }

            list($vinculom, $vinculoh) = $this->vinculoNovo($nrpessoa, Util::datadeString($vinculo['DTADMISSAOVINC'],true), $vinculo['NRVINCULOM'], $vinculo['NRTIPOVINCULOM'], $vinculo['NRTPADMISSAO'] ?? null,
                                            $vinculo['NRVINCULOEMPREG'], Util::datadeString($vinculo['DTOPCAOFGTS'],true), Util::datadeString($vinculo['DTAPOSENTAFGTS'],true), Util::datadeString($vinculo['DTPRIMADMISSAO'],true),$vinculo['NMVINCULOM'],
                                            $vinculo['NRDIASCONTRATOEXP'], $vinculo['NRDIASCONTRATOEXPPRO'], Util::datadeString($vinculo['DTEXAMEMEDICO'],true), $competencia, $parametros['NRSITUFUNCM'],
                                            $parametros['NROCUPACAOM'], $parametros['NRESCALATRABM'], $vinculo['IDCONTRIBUISIND'], 0, 0, Util::datadeString($vinculo['DTBASEFERIAS'],true),
                                            $vinculo['IDREMUNERACAO'], $vinculo['IDTPPAGAMENTO'], $movimentacao['NRESTRUTLEGAL'], $movimentacao['NRESTRUTGEREN'], $parametros['NRTPMODALIDSAL'], $vinculo['IDMULTVINC'],
                                            $parametros['NROCUPACAOM'], $vinculo['CDCONTRIBINDIVIDUAL'], $vinculo['NRTPMOVTRANSFM'], Util::datadeString($vinculo['DTFIMCONTRDETERMIN'],true), $movimentacao['NRESTRUTSIND'],
                                            'N', $this->nrorg, null, $vinculo['DSLIVROADMISSAO'], Util::datadeString($vinculo['DTENTREGAEPI'], true), $movimentacao['CDMATRICULAESOCIAL'], $vinculo['DTVENCCONTREXPER'], $vinculo['NRCLAUSASSECURAT'], $vinculo['IDPAGPROPFER13'],
                                            $vinculo['NRMOTIVOADMISSAO'], $vinculo['NRVINCULOMSUBST'], null, $vinculo['IDPARTARTCLT']);

            if(!$vinculom || !$vinculoh)
                throw new \Exception("Falha na criação do vínculo.");

            if ( $anoAtual > $anoAdmissao ) {
                if ( $anoCompetencia === $anoAtual) {
                    $this->novoVinculoHistorico(
                        $this->nrorg, $vinculom->getNrvinculom(), Util::datadeString($this->hcmParameters->getDtCompetencia(),true), $parametros['NRSITUFUNCM'], $vinculoh->getNrcargo(),
                        $vinculoh->getNrescalatrabm(), $vinculoh->getIdcontribuisind(), 0, 0, Util::datadeString($vinculo['DTBASEFERIAS'],true), $vinculoh->getIdremuneracao(),
                        $vinculoh->getIdtppagamento(), $vinculoh->getNrestrutlegal(), $vinculoh->getNrestrutgeren(), $vinculoh->getNrtpmodalidsal(), $vinculo['IDMULTVINC'],
                        $vinculoh->getNrfuncao(), $vinculoh->getCdcontribindividual(), GpeTpmovtransfm::TPMOVTRANSF_ADMIS_EMPR_ANT, null,
                        $vinculoh->getNrestrutsind(), 'N', null, null
                    );
                } else if ( $anoCompetencia < $anoAtual ) {
                    $dtComp = "01/"."01/".$anoCompetencia;
                    for ( $i = $anoCompetencia; $i <= $anoAtual; $i++ ) {
                        $this->novoVinculoHistorico(
                            $this->nrorg, $vinculom->getNrvinculom(), Util::datadeString($dtComp,true), $parametros['NRSITUFUNCM'], $vinculoh->getNrcargo(),
                            $vinculoh->getNrescalatrabm(), $vinculoh->getIdcontribuisind(), 0, 0, Util::datadeString($vinculo['DTBASEFERIAS'],true), $vinculoh->getIdremuneracao(),
                            $vinculoh->getIdtppagamento(), $vinculoh->getNrestrutlegal(), $vinculoh->getNrestrutgeren(), $vinculoh->getNrtpmodalidsal(), $vinculo['IDMULTVINC'],
                            $vinculoh->getNrfuncao(), $vinculoh->getCdcontribindividual(), GpeTpmovtransfm::TPMOVTRANSF_ADMIS_EMPR_ANT, null,
                            $vinculoh->getNrestrutsind(), 'N', null, null
                        );
                        $anoCompetencia++;
                        $dtComp = "01/"."01/".$anoCompetencia;
                    }
                }
            }

            // INSERE MOVIMENTAÇÕES            
            if(!empty($movimentacao['NRESTRUTSIND'])) {
                $this->incluiMovimentacao($this->nrorg,$movimentacao['NRTIPOTRANSFER_SINDICAL'],$vinculom->getNrvinculom(),$movimentacao['NRESTRUTSIND'],Util::datadeString($movimentacao['DTINIMOVIMENT'],true), 10, $movimentacao['NRTPMOVTRANSF']);
            }
            $this->incluiMovimentacao($this->nrorg,$movimentacao['NRTIPOTRANSFER_LEGAL'],$vinculom->getNrvinculom(), $movimentacao['NRESTRUTLEGAL'],Util::datadeString($movimentacao['DTINIMOVIMENT'],true), $this->hcmParameters->getTipoEstruturaLegalOrg(), $movimentacao['NRTPMOVTRANSF']);
            $this->incluiMovimentacao($this->nrorg,$movimentacao['NRTIPOTRANSFER_GEREN'],$vinculom->getNrvinculom(), $movimentacao['NRESTRUTGEREN'],Util::datadeString($movimentacao['DTINIMOVIMENT'],true), $this->hcmParameters->getTipoEstruturaGerencialOrg(), $movimentacao['NRTPMOVTRANSF']);

            // INSERE SITUAÇÃO
            $this->altesitufuncNova($this->nrorg,$vinculom->getNrvinculom(),$parametros['NRSITUFUNCM'], Util::datadeString($parametros['DTINISITUFUNC'],true));

            // INSERE SALARIO
            $this->alteSalarioNova($this->nrorg,$vinculom->getNrvinculom(),Util::datadeString($parametros['DTALTESALARIO'],true), $parametros['NROCORRENCIA'], $parametros['NRTIPOALTERA'], $this->validaValorMonetario($parametros['VRSALARIO']), $parametros['CDTIPOSALARIO'],
                                    $parametros['NRTPMODALIDSAL'],null,$this->validaValorMonetario($parametros['VRDIFERENCAREAL']),$this->validaValorMonetario($parametros['VRPERCREAL']),$this->validaValorMonetario($parametros['VRPERCALTERA']),$this->validaValorMonetario($parametros['VRSALARIOHR']));
            
            // INSERE ESCALA
            $this->alteEscalaNova($this->nrorg,$vinculom->getNrvinculom(),$parametros['NRESCALATRABM'], Util::datadeString($parametros['DTINIESCALA'],true),null,Util::datadeString($parametros['DTFIMESCALA'],true), null, null, $parametros['NRTURNO'],$parametros['QTHRLIMITESEPHEXTRA']);
            
            // INSERE OCUPAÇÃO
            $this->alteOcupacaoNova($this->nrorg,$vinculom->getNrvinculom(),$parametros['NROCUPACAOM'], Util::datadeString($parametros['DTINIOCUPACAO'],true), $parametros['NRTIPOOCUPACAO'], $parametros['NRTPMODALIDSAL'],null,null,Util::datadeString($parametros['DTFIMOCUPACAO'],true));

            //INSERE DEPENDENTES DO PARCEIRO
            if(!empty($arrayDependente)){
                $benefDeps = [];
                foreach ($arrayDependente as $dependentes) {
                    $dependentes = $this->verificaIndex($dependentes);
                    $dependentes['NMPESSOA'] = Util::removeCaracteresEspeciais($dependentes['NMPESSOA']);
                    $dependentes['NMPESSOA'] = strtoupper($dependentes['NMPESSOA']);
                    $dependentes['NRCPFPESSOA'] = str_replace(array('.','/','-'),'',$dependentes['NRCPFPESSOA']);
                    $dtnascdependente = isset($dependentes['DTNASCIFUNDPARC']) ? Util::datadeString($dependentes['DTNASCIFUNDPARC'],true) : null;
                    
                    //INSERE PARCNEGOCIO DO DEPENDENTE
                    $parcnegocioDep = $this->logicpessoa->parcNegocioNovo($this->nrorg, $dependentes['NMPESSOA'], $dependentes['NMPESSOA'],'PESSOA','S','N','CPF', $dependentes['NRCPFPESSOA'], $dtnascdependente);
                    
                    //INSERE PESSOA DO DEPENDENTE
                    $pessoaDep = $this->logicpessoa->pessoaNova($this->nrorg, $parcnegocioDep->getNrparcnegocio(), $newdtmescompetenc, $dependentes['NMPESSOA'], null, null, $dtnascdependente,
                                                                $dependentes['IDSEXOPESSOA'], null, $dependentes['NRCPFPESSOA'], null, null, null, null, null, null,
                                                                null, null, $dependentes['CDESTACIVIL'], null, null,null, null, null, null, null,
                                                                null, null, null, null,null, null, null,null, null, null, null, $dependentes['CDPAIS'], $dependentes['SGESTADO'],
                                                                $dependentes['CDMUNICIPIO'], null, null, null, null, null,$dependentes['NRNACIONALIDADE'], null);

                    //INSERE RELACIONAPARC
                    $this->novoRelacionaparc($this->nrorg, $dependentes['NRTIPORELACIONA'], $nrparcnegocio, Util::datadeString($dependentes['DTFIMRELACIONAMENTO'],true), $parcnegocioDep->getNrparcnegocio());
                }
            }
            
            //INSERE PROGRAMAÇÃO OCORRÊNCIA DE FREQUÊNCIA
            $this->insereProgOcorrenciaFrequencia($this->nrorg, $vinculo['NRTIPOVINCULOM'], $vinculom->getNrvinculom(), $vinculom->getDtadmissaovinc(), $vinculoh->getNrestrutlegal(), $vinculoh->getNrestrutgeren(), $vinculoh->getNrestrutsind(), $parametros['NROCUPACAOM']);
            
            $this->setParametrosAdmissao($vinculo['NRTIPOVINCULOM'], $vinculoh->getNrestrutlegal(), $vinculoh->getNrestrutgeren(), $vinculoh->getNrestrutsind(), $parametros['NROCUPACAOM']);
            
            //INSERE BANCO DE HORAS
            $this->insereBancoHorasAdmissao($this->nrorg, $vinculo['NRTIPOVINCULOM'], $vinculom->getNrvinculom(), $vinculom->getDtadmissaovinc(), $vinculoh->getNrestrutlegal(), $vinculoh->getNrestrutgeren(), $vinculoh->getNrestrutsind(), $parametros['NROCUPACAOM']);
            
            //INSERE OS PERFIS DE ACESSO DO NOVO OPERADOR
            if (($operadorFoiCriado) && !empty($nrparcnegocio)) {
                $this->setParametrosAdmissaoPerfisAcesso($nrparcnegocio);
            }

            return true;

        } catch (\Exception $e) {
            $msgError = $e->getMessage();

            $arrayMsgError[] = "Ocorreu um erro ao criar dados do vínculo. Error: $msgError";

            if($this->connection->isTransactionActive()) {
                $this->entityManager->rollback();
            }
        }
    }
    
    private function setaDados() {
        return [$this->arrDados['DTADMISSAOVINC'],    $this->arrDados['IDMULTVINC'],           $this->arrDados['NRPESSOA'],            $this->arrDados['NEWDTMESCOMPETENC'], $this->arrDados['NMVINCULOM'],
                $this->arrDados['NRESTRUTLEGAL'],     $this->arrDados['NRVINCULOM'],           $this->arrDados['NRTIPOVINCULOM'],      $this->arrDados['NRTPADMISSAO'],      $this->arrDados['NRVINCULOEMPREG'],
                $this->arrDados['NRDIASCONTRATOEXP'], $this->arrDados['NRDIASCONTRATOEXPPRO'], $this->arrDados['DTEXAMEMEDICO'],       $this->arrDados['NRSITUFUNCM'],       $this->arrDados['NROCUPACAOM'],
                $this->arrDados['NRESCALATRABM'],     $this->arrDados['IDCONTRIBUISIND'],      $this->arrDados['DTBASEFERIAS'],        $this->arrDados['IDREMUNERACAO'],     $this->arrDados['IDTPPAGAMENTO'],
                $this->arrDados['NRESTRUTGEREN'],     $this->arrDados['NRTPMODALIDSAL'],       $this->arrDados['CDCONTRIBINDIVIDUAL'], $this->arrDados['NRTPMOVTRANSFM'],    $this->arrDados['DTFIMCONTRDETERMIN'],
                $this->arrDados['NRESTRUTSIND'],      $this->arrDados['VRSALARIO'],            $this->arrDados['NMPESSOAPAI'],         $this->arrDados['NMPESSOAMAE'],       $this->arrDados['NRPARCNEGOCIO'],
                $this->arrDados['OPERADORFOICRIADO'], $this->arrDados['DTOPCAOFGTS'],          $this->arrDados['DTAPOSENTAFGTS'],      $this->arrDados['DTPRIMADMISSAO']];
    }

    private function verificaIndex($indexs){
        $arrayindex = ['NMPESSOA','DSAPELIDOPESS','NRCERTNASCPES','DTNASCPESSOA','IDSEXOPESSOA','NRCONDFISPES','NRCPFPESSOA','NRCTPSPESSOA',
                       'NRSERIECTPSPES','SGUFCTPSPES','DTCTPSPESSOA','NRPISPASEPPES','DTINSCPISPASEP','NRINSCINSSPES','NRINSCISSPES','CDESTACIVIL','NRCERTCASAPES',
                       'DTCASAPESSOA','IDISENTITUELEI','NRTITUELEIPES','NRSECAELEIPES','NRZONAELEIPES','NRCARTHABPES','DTHABCARTPES','DTVALHABCARTPES','DSCATEGHABCART',
                       'NRCATMILITARPE','NRCERTRESEPES','DSCATEGCERTRESE','DTCERTRESEPES','CDEXCERTRESEPES','NRRACAPESSOA','IDGRUPSANGUPES','IDFATORRHPES','CDPAIS',
                       'SGESTADO','CDMUNICIPIO','NRRGPESSOA','CDEXRGPESSOA','SGUFRGPESSOA','DSLOCALEXRGPES','DTEXRGPESSOA','NRNACIONALID','NRGRAUINSTR','DSPRONOMETRAT',
                       'NRANOCHEGAPAIS','NRDOCUESTRANG','IDPESNATURALIZA','DTNATURALIZAPES','IDPOSVISTOPERM','DTPERMAVISTO','NRCONSELHOREG','NRINSCRCONSREG',
                       'DTEXCONSREG','SGUFCONSREG','NRLIVROCERTNASC','NRFOLHACERTNASC','DSCOMPLCERTNASC','NRREGCTPSPES','CDLOGRADOURO','NMBAIRROENDERECO',
                       'CDTIPOENDERECO','NRCEPENDERECO','DSCOMPLEENDERECO','NRIMOVELENDERECO','DSENDERECO','DSREFERENCIAENDE','CDBANCO',
                       'CDBANCO','CDAGENCIA','CDCONTCORR','NRTPCONTCORR','EMAIL','NRPESSOA','DTADMISSAOVINC','CDMATRICULA','NRTIPOVINCULOM',
                       'NRVINCULOEMPREG','DTOPCAOFGTS','DTAPOSENTAFGTS','DTPRIMADMISSAO','NMVINCULOM','NRDIASCONTRATOEXP','NRDIASCONTRATOEXPPRO','DTEXAMEMEDICO',
                       'IDCONTRIBUISIND','DTBASEFERIAS','IDREMUNERACAO','IDTPPAGAMENTO','IDMULTVINC','CDCONTRIBINDIVIDUAL','NRTPMOVTRANSFM','DTFIMCONTRDETERMIN',
                       'NRTIPOTRANSFER_SINDICAL','NRESTRUTSIND','DTINIMOVIMENT','NRTIPOTRANSFER_LEGAL','NRESTRUTLEGAL','NRTIPOTRANSFER_GEREN','NRESTRUTGEREN', 'NRTIPOTRANSFER_ADICIONAL', 'NRESTRUTADICIONAL',
                       'NRTPMOVTRANSF','NRSITUFUNCM','DTINISITUFUNC','QTHRLIMITESEPHEXTRA','QTHRLIMITESEPHEXTRA','DTALTESALARIO','NROCORRENCIA','NRTIPOALTERA','VRSALARIO',
                       'CDTIPOSALARIO','NRTPMODALIDSAL','VRDIFERENCAREAL','VRPERCREAL','VRPERCALTERA','NRESCALATRABM','DTINIESCALA','DTFIMESCALA','NRTURNO','QTHRLIMITESEPHEXTRA',
                       'NROCUPACAOM','DTINIOCUPACAO','NRTIPOOCUPACAO','NRTPMODALIDSAL','DTFIMOCUPACAO','INSERE_OPERADOR','NRTIPORELACIONA','DTFIMRELACIONAMENTO','NRTIPODEPENDE','NRNACIONALIDADE',
                       'NRCLASSESTRANG', 'DSEXPDOCESTR', 'DTCHEGADAPAIS', 'NRMOTIVOADMISSAO', 'NRVINCULOMSUBST', 'CDMUNICIPIO_END', 'NEWDTMESCOMPETENC', 'NRPESSOA', 'DSLIVROADMISSAO','DTENTREGAEPI','CDMATRICULAESOCIAL',
                       'DTVENCCONTREXPER', 'NRCLAUSASSECURAT', 'IDPAGPROPFER13', 'NRMOTIVOADMISSAO'
                    ];

        foreach($arrayindex as $index){
            if(!isset($indexs[$index]) || empty($indexs[$index])){
                $indexs[$index] = null;
            }
        }
        return $indexs;
    }

    private function anoData($data) {
        $data = explode("/", $data);
        list($diaAtual, $mesAtual, $anoAtual) = $data;

        return $anoAtual;
    }

    private function vinculoNovo($nrPessoa, $dtAdmissaoVinculo, $cdMatricula, $nrTipoVinculom, $nrTipoAdmissao,
                                 $nrVinculoEmpregaticio, $dtOpcaoFGTS, $dtAposentadoriaFGTS, $dtPrimeiraAdmissao,$nmVinculom,
                                 $nrDiasContratoExp, $nrDiasContratoExpPro, $dtExameMedico, $dtmescompetenc, $nrsitufuncm,
                                 $nrcargo, $nrescalatrabm, $idcontribuisind, $nrdependir, $nrdependsfam, $dtbaseferias,
                                 $idremuneracao, $idtppagamento, $nrestrutlegal, $nrestrutgeren, $nrtpmodalidsal, $idmultvinc,
                                 $nrfuncao, $cdcontribindividual, $nrtpmovtransfm, $dtfimcontrdetermin, $nrestrutsind,
                                 $confidencial = null,$nrorg = null,$dtRescisao = null, $dslivroadmissao = null, $dtentregaepi = null, 
                                 $cdmatriculaEsocial = null, $dtvenccontrexper = null, $nrclausassecurat = null, $idpagpropfer13 = null,
                                 $nrmotivoadmissao = null, $nrvinculomsubst = null, $idAssocSindicato = null, $idpartartclt = null){
        
        $vinculom = $this->novoVinculoMestre($nrPessoa, $nrorg, $dtAdmissaoVinculo, $dtRescisao, $cdMatricula, $nrTipoVinculom,
                        $nrVinculoEmpregaticio, $nrTipoAdmissao, $nmVinculom, $dtPrimeiraAdmissao, $dtOpcaoFGTS,
                        $dtExameMedico, $dtAposentadoriaFGTS, $nrDiasContratoExp, $nrDiasContratoExpPro, $dslivroadmissao, $dtentregaepi,
                        $dtvenccontrexper, $nrclausassecurat, $nrmotivoadmissao, $nrvinculomsubst);
        
        $vinculoh = $this->novoVinculoHistorico(
                        $nrorg,$vinculom->getNrvinculom(), $dtmescompetenc, $nrsitufuncm, $nrcargo, $nrescalatrabm,
                        $idcontribuisind, $nrdependir, $nrdependsfam, $dtbaseferias, $idremuneracao, $idtppagamento, $nrestrutlegal,
                        $nrestrutgeren, $nrtpmodalidsal, $idmultvinc, $nrfuncao, $cdcontribindividual, $nrtpmovtransfm,
                        $dtfimcontrdetermin, $nrestrutsind, $confidencial, $cdmatriculaEsocial, $idpagpropfer13, $idAssocSindicato, $idpartartclt
                    );

        return [$vinculom, $vinculoh];
    }

    private function novoVinculoMestre(
                        $nrpessoa, $nrorgTrab, $dtAdmissao, $dtRescisao = null, $matricula = null, $tipoVinculo = null,
                        $nrvinculoempreg = null, $nrtpadmissao = null, $nmvinculom = null, $dtPrimeiraAdmissao = null, $dtOpcaoFGTS = null,
                        $dtExameMedico = null, $dtAposentadoriaFGTS = null, $nrDiasContratoExp = null, $nrDiasContratoExpPro = null, 
                        $dslivroadmissao = null, $dtentregaepi = null, $dtvenccontrexper = null, $nrclausassecurat = null,
                        $nrmotivoadmissao = null, $nrvinculomsubst = null
    ){
        try{
            $vinculo = new $this->gpe_Vinculom;
            $vinculo->setNrvinculom(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_VINCULOM', 1, 12, true));
            
            if(empty($matricula)){
                $matricula = $vinculo->getNrvinculom();
            }
        
            $vinculo->setNrpessoa($nrpessoa);
            $vinculo->setNrorg($nrorgTrab);
            $vinculo->setDtadmissaovinc($dtAdmissao);
            $vinculo->setDtrescisaovinc($dtRescisao);
            $vinculo->setCdmatricula($matricula);
            $vinculo->setNrtipovinculom($tipoVinculo);
            $vinculo->setNrvinculoempreg($nrvinculoempreg);
            $vinculo->setNrtpadmissao($nrtpadmissao);
            $vinculo->setNmvinculom($nmvinculom);
            $vinculo->setDtinclusao(DateUtil::getDataAtual());
            $vinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $vinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $vinculo->setDtultatu(DateUtil::getDataAtual());
            $vinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $vinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $vinculo->setDtprimadmissao($dtPrimeiraAdmissao);
            $vinculo->setDtopcaofgts($dtOpcaoFGTS);
            $vinculo->setDtaposentafgts($dtAposentadoriaFGTS);
            $vinculo->setNrdiascontratoexp($nrDiasContratoExp);
            $vinculo->setNrdiascontratoexppro($nrDiasContratoExpPro);
            $vinculo->setDtexamemedico($dtExameMedico);
            $vinculo->setDslivroadmissao($dslivroadmissao);
            $vinculo->setDtentregaepi($dtentregaepi);
            $vinculo->setDtvenccontrexper(DateUtil::getDataDeString($dtvenccontrexper));
            $vinculo->setNrclausassecurat($nrclausassecurat);
            $vinculo->setNrmotivoadmissao($nrmotivoadmissao);
            $vinculo->setNrvinculomsubst($nrvinculomsubst);
        
            $this->entityManager->persist($vinculo);
            $this->entityManager->flush($vinculo);
            
            return $vinculo;
        }
        catch (Exception $e){
            $e = new Exception('Erro ao criar um novo vinculo. Nao pode inserir um novo mestre.');
            throw $e;
        }
    }

    private function novoVinculoHistorico($nrorg, $nrvinculom, $dtmescompetenc, $nrsitufuncm, $nrcargo, $nrescalatrabm,
                                            $idcontribuisind, $nrdependir, $nrdependsfam, $dtbaseferias, $idremuneracao,
                                            $idtppagamento, $nrestrutlegal, $nrestrutgeren, $nrtpmodalidsal, $idmultvinc,
                                            $nrfuncao, $cdcontribindividual, $nrtpmovtransfm, $dtfimcontrdetermin,
                                            $nrestrutsind, $confidencial, $cdmatriculaEsocial = null, $idpagpropfer13 = null, $idAssocSindicato = 'N', $idpartartclt = 'N')
    {
        /** @var \Teknisa\Model\Gestaopessoas\Entities\GpeVinculoh $vinculoh */
        $vinculoh = new $this->gpe_Vinculoh;

        $vinculoh->setNrorg($nrorg);
        $vinculoh->setNrvinculom($nrvinculom);
        $vinculoh->setDtmescompetenc($dtmescompetenc);

        $vinculoh->setDtbaseferias($dtbaseferias);
        $vinculoh->setIdmultvinc($idmultvinc);
        $vinculoh->setNrsitufuncm($nrsitufuncm);
        $vinculoh->setNrtpmovtransfm($nrtpmovtransfm);
        $vinculoh->setCdcontribindividual($cdcontribindividual);
        $vinculoh->setDtfimcontrdetermin($dtfimcontrdetermin);

        $vinculoh->setNrcargo($nrcargo);
        $vinculoh->setNrfuncao($nrfuncao);
        $vinculoh->setNrescalatrabm($nrescalatrabm);
        $vinculoh->setIdremuneracao($idremuneracao);
        $vinculoh->setIdtppagamento($idtppagamento);
        $vinculoh->setNrtpmodalidsal($nrtpmodalidsal);

        $vinculoh->setNrdependir($nrdependir);
        $vinculoh->setNrdependsfam($nrdependsfam);
        $vinculoh->setConfidencial($confidencial);

        $vinculoh->setNrestrutlegal($nrestrutlegal);
        $vinculoh->setNrestrutgeren($nrestrutgeren);
        $vinculoh->setNrestrutsind($nrestrutsind);
        $vinculoh->setIdcontribuisind(($idcontribuisind) ? : 'NUNCA');
        $vinculoh->setCdmatriculaesocial($cdmatriculaEsocial);
        $vinculoh->setIdpagpropfer13($idpagpropfer13);
        $vinculoh->setIdassocsindicato($idAssocSindicato);
        $vinculoh->setIdpartclt($idpartartclt);
        
        $vinculoh->setDtinclusao(DateUtil::getDataAtual());
        $vinculoh->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $vinculoh->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $vinculoh->setDtultatu(DateUtil::getDataAtual());
        $vinculoh->setCdoperultatu($this->hcmParameters->getCdoperador());
        $vinculoh->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());

        $vinculoh->setNrvinculoh(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_VINCULOH', 1, 12, true));
        
        $this->entityManager->persist($vinculoh);
        $this->entityManager->flush($vinculoh);
        
        return $vinculoh;
    }

    /**
     * Inclui movimentacao do vinculo.
     *
     * @param integer    $nrOrg            Numero da Organizacao.
     * @param integer    $nrTipoTransfer   Numero do Tipo de transferencia.
     * @param integer    $nrVinculom       Numero do Vinculo.
     * @param integer    $nrEstruturam     Numero da Estrutra.
     * @param string     $dtIniMoviment    Data inicial da movimentacao do vinculo.
     * @param integer    $nrTipoEstrutura  Numero do Tipo de Estrutura.
     * @param integer    $nrTpMovTransf    Numero do Tipo de Movimentacao de Transferencia.
     * @param DateTime   $dtRetroativa     Data Retroativa.
     * @param string     $idAtualizaAcerto Identificar de atualizacao de acerto (normal = null, excluir = 1 ou incluir = 2).
     * @param string     $dtFimMovimento   Data final da movimentacao do vinculo.
     *
     * @return $this->gpeMovimentacao.
     */
    private function incluiMovimentacao(
        $nrOrg, $nrTipoTransfer, $nrVinculom, $nrEstruturam, $dtIniMoviment, $nrTipoEstrutura = null,
        $nrTpMovTransf = null, $nrMotivoTransf = null, $dtRetroativa = null, $idAtualizaAcerto = 0,
        $dtFimMovimento = null, $dsObservacao = null
    ){
        $movimentacao = new $this->gpeMovimentacao;
        $movimentacao->setNrorg($nrOrg);
        $movimentacao->setNrtipotransfer($nrTipoTransfer);
        $movimentacao->setNrvinculom($nrVinculom);
        $movimentacao->setNrestruturam($nrEstruturam);
        $movimentacao->setDtinimoviment($dtIniMoviment);
        $movimentacao->setNrtipoestrutura($nrTipoEstrutura);
        $movimentacao->setNrtpmovtransfm($nrTpMovTransf);
        $movimentacao->setNrmotivotransf($nrMotivoTransf);
        $movimentacao->setDtmovimentretro($dtRetroativa);
        $movimentacao->setIdatualizaacerto($idAtualizaAcerto);
        $movimentacao->setDtfimmoviment($dtFimMovimento);
        $movimentacao->setDsobservacao($dsObservacao);

        $movimentacao->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $movimentacao->setDtinclusao(DateUtil::getDataAtual());
        $movimentacao->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $movimentacao->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $movimentacao->setDtultatu(DateUtil::getDataAtual());
        $movimentacao->setCdoperultatu($this->hcmParameters->getCdoperador());

        $movimentacao->setNrmovimentacao(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_MOVIMENTACAO', 1, 12, true));

        $this->entityManager->persist($movimentacao);
        $this->entityManager->flush();

        // Retorna movimentacao
        return $movimentacao;
    }

    private function altesitufuncNova($nrorg, $nrvinculom, $nrSituacaoFuncionario, $dtIniSituacaoFunc,
                                      $motivoAfastamento = null, $cdDiagnostico = null, $codigoCDI = null,
                                      $dtFimSituacaoFunc = null, $dtPreviretoSitu = null, $dtProrrSituacaoFunc = null,
                                      $dtVerifSituacaoFunc = null, $nrProxSituFunc = null, $qntDias = null,
                                      $dsObs = null, $nrpessoa = null)
    {
        /** @var \Teknisa\Model\Gestaopessoas\Entities\GpeAltesitufunc $altesitufunc */
        
        $altesitufunc = new $this->gpeAltesitufunc;
        
        $altesitufunc->setNrorg($nrorg);
        $altesitufunc->setNrvinculom($nrvinculom);
        $altesitufunc->setNrsitufuncm($nrSituacaoFuncionario);
        $altesitufunc->setDtinisitufunc($dtIniSituacaoFunc);
        $altesitufunc->setNrqtdiasitufunc($qntDias);

        $altesitufunc->setNrmotivoafasta($motivoAfastamento);
        $altesitufunc->setCddiagnost($cdDiagnostico);
        $altesitufunc->setCdtabecdi($codigoCDI);
        $altesitufunc->setDtfimsitufunc($dtFimSituacaoFunc);
        $altesitufunc->setDtpreviretositu($dtPreviretoSitu);
        $altesitufunc->setDtprorrsitufunc($dtProrrSituacaoFunc);
        $altesitufunc->setDtverifsitufunc($dtVerifSituacaoFunc);
        $altesitufunc->setNrproxsitufunc($nrProxSituFunc);
        $altesitufunc->setDsObsAltSitFunc($dsObs);
        $altesitufunc->setNrpessoa($nrpessoa);

        $altesitufunc->setNrorginclusao($nrorg);
        $altesitufunc->setDtinclusao(DateUtil::getDataAtual());
        $altesitufunc->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $altesitufunc->setNrorgultatu($nrorg);
        $altesitufunc->setDtultatu(DateUtil::getDataAtual());
        $altesitufunc->setCdoperultatu($this->hcmParameters->getCdoperador());

        $altesitufunc->setNraltesitufunc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESITUFUNC', 1, 12, true));

        $this->entityManager->persist($altesitufunc);
        return $altesitufunc;
    }

    private function validaValorMonetario($valor) {
        if(!empty($valor)){
            $valor = str_replace(',','.',$valor);
            return $valor;
        }
        return null;
    }
    
    private function alteSalarioNova($nrorg,$nrvinculom, $dtAlteracaoSalario, $nrOcorrencia, $nrTipoAlteracao, $vrSalario,
                                     $idTipoSalario, $nrTipoModalideSalario, $observacao = null, $dtbasedissidio = null, $vrDiferencaReal = null, $vrPercentualReal = null,
                                     $vrpercaltera = null, $vrsalariohr = null)
    {
        $alteSalario = new $this->gpe_Altesalario;
        $alteSalario->setNrvinculom($nrvinculom);
        $alteSalario->setDtaltesalario($dtAlteracaoSalario);
        $alteSalario->setDtbasedissidio($dtbasedissidio);
        $alteSalario->setNrocorrencia($nrOcorrencia);
        $alteSalario->setNrtipoaltera($nrTipoAlteracao);
        $alteSalario->setVrpercaltera($vrpercaltera);
        $alteSalario->setVrsalario($vrSalario);
        $alteSalario->setVrsalariohr($vrsalariohr);
        $alteSalario->setNrorg($nrorg);
        $alteSalario->setIdtpsalario($idTipoSalario);
        $alteSalario->setNrtpmodalidsal($nrTipoModalideSalario);
        $alteSalario->setDsobservacao($observacao);

        $alteSalario->setVrdiferencareal($vrDiferencaReal);
        $alteSalario->setVrpercreal($vrPercentualReal);

        $alteSalario->setDtinclusao(DateUtil::getDataAtual());
        $alteSalario->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $alteSalario->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $alteSalario->setDtultatu(DateUtil::getDataAtual());
        $alteSalario->setCdoperultatu($this->hcmParameters->getCdoperador());
        $alteSalario->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        
        
        $alteSalario->setNraltesalario(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESALARIO', 1, 12, true));
        
        $this->entityManager->persist($alteSalario);
        $this->entityManager->flush($alteSalario);
    }

    private function alteEscalaNova($nrorg, $nrvinculom, $nrEscalaTrabalho, $dtIniEscala, $observacao = null,
                         $dtFimEscala = null, $nrhordiam = null, $nrMotivoEscala = null, $nrturno = null,$qthrlimitesephextra = null)
    {
        $alteEscala = new $this->gpe_alteEscala;
        $alteEscala->setNrvinculom($nrvinculom);
        $alteEscala->setNrorg($nrorg);
        $alteEscala->setNrescalatrabm($nrEscalaTrabalho);
        $alteEscala->setDtiniescala($dtIniEscala);
        $alteEscala->setNrmotivoescala($nrMotivoEscala);
        $alteEscala->setNrturno($nrturno);
        $alteEscala->setDtfimescala($dtFimEscala);
        $alteEscala->setNrhordiam($nrhordiam);
        $alteEscala->setDsobservacao($observacao);
        $alteEscala->setQthrlimitesephextra($qthrlimitesephextra);

        $alteEscala->setDtinclusao(DateUtil::getDataAtual());
        $alteEscala->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $alteEscala->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $alteEscala->setDtultatu(DateUtil::getDataAtual());
        $alteEscala->setCdoperultatu($this->hcmParameters->getCdoperador());
        $alteEscala->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());

        $alteEscala->setNralteescala(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTEESCALA', 1, 12, true));
        
        $this->entityManager->persist($alteEscala);
        $this->entityManager->flush($alteEscala);
    }


    private function alteOcupacaoNova($nrorg,$nrvinculom, $nrOcupacao, $dtIniOcupacao, $nrTipoOcupacao, $nrTpModalidade,
                                     $nrMotivoOcupacao = null, $nrNivelOcupacao = null, $dtFimOcupacao = null, $observacao = null)
    {
        $alteOcupacao = new $this->gpe_alteOcupacao;

        $alteOcupacao->setNrorg($nrorg);
        $alteOcupacao->setNrvinculom($nrvinculom);
        $alteOcupacao->setNrocupacaom($nrOcupacao);
        $alteOcupacao->setDtiniocupacao($dtIniOcupacao);

        $alteOcupacao->setNrtipoocupacao($nrTipoOcupacao);

        $alteOcupacao->setNrmotivoocupa($nrMotivoOcupacao);
        $alteOcupacao->setNrnivelocupa($nrNivelOcupacao);
        $alteOcupacao->setNrtpmodalidsal($nrTpModalidade);
        $alteOcupacao->setDsobservacao($observacao);
        $alteOcupacao->setDtfimocupacao($dtFimOcupacao);

        $alteOcupacao->setDtinclusao(DateUtil::getDataAtual());
        $alteOcupacao->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $alteOcupacao->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $alteOcupacao->setDtultatu(DateUtil::getDataAtual());
        $alteOcupacao->setCdoperultatu($this->hcmParameters->getCdoperador());
        $alteOcupacao->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());

        $alteOcupacao->setNralteocupacao(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTEOCUPACAO', 1, 12, true));

        $this->entityManager->persist($alteOcupacao);
        $this->entityManager->flush($alteOcupacao);
    }

    private function insereProgOcorrenciaFrequencia($nrorg, $nrtipovinculom, $nrvinculom, $dtadmissao, $nrestrutlegal, $nrestrutgeren, $nrestrutsind, $nrocupacaom) {
        try{
            $arrayProgOcorrencia = $this->entityManager->getRepository($this->pto_ocorrenciadmissao)->getOcorrenciasNaAdmissao(
                $nrorg, $nrtipovinculom, $nrestrutlegal, $nrestrutgeren, $nrestrutsind, $nrocupacaom
            );
            
            foreach($arrayProgOcorrencia as $progOcorrencia) {
                $itemProgOcorrencia = new $this->pto_progocorrencia;
                $itemProgOcorrencia->setNrorg($nrorg);
                $itemProgOcorrencia->setNrprogocorrencia(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PTO_PROGOCORRENCIA', 1, 12, true));
                $itemProgOcorrencia->setNrvinculom($nrvinculom);
                $itemProgOcorrencia->setNrtipoprogocorrencia($progOcorrencia['NRTIPOPROGOCORRENCIA']);
                $itemProgOcorrencia->setDtinicioprogocor($dtadmissao);
                $itemProgOcorrencia->setIdprogocorconfirmada('S');
                //$itemProgOcorrencia->setDsobservacao('Ocorrência padrão da admissão.');
                $itemProgOcorrencia->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $itemProgOcorrencia->setCdoperultatu($this->hcmParameters->getCdoperador());
                $itemProgOcorrencia->setDtinclusao(DateUtil::getDataAtual(false));
                $itemProgOcorrencia->setDtultatu(DateUtil::getDataAtual(false));
                $itemProgOcorrencia->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $itemProgOcorrencia->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                
                $this->entityManager->persist($itemProgOcorrencia);
                $this->entityManager->flush($itemProgOcorrencia);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Define os parametros a serem considerados na admissao corrente
     * @var string $nrtipovinculom
     * @var string $nrestrutlegal
     * @var string $nrestrutgeren
     * @var string $nrestrutsind
     * @var string $nrocupacaom
     * @return array
     */
    private function setParametrosAdmissao(
        ?string $nrtipovinculom,
        ?string $nrestrutlegal,
        ?string $nrestrutgeren,
        ?string $nrestrutsind,
        ?string $nrocupacaom
    ): array {
        return $this->parametrosAdmissao = $this->entityManager->getRepository($this->pto_bancohorasadmi)->getBancoHorasNaAdmissao(
            $this->hcmParameters->getNrorgTrabalho(), $nrtipovinculom, $nrestrutlegal, $nrestrutgeren, $nrestrutsind, $nrocupacaom
        );
    }

    private function insereBancoHorasAdmissao($nrorg, $nrtipovinculom, $nrvinculom, $dtadmissao, $nrestrutlegal, $nrestrutgeren, $nrestrutsind, $nrocupacaom) {
        try{
            foreach($this->parametrosAdmissao as $parametro){
                $itemBancoHoras = new $this->pto_altebancohoras;
                $itemBancoHoras->setNrorg($nrorg);
                $itemBancoHoras->setNraltebancohoras(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PTO_ALTEBANCOHORAS', 1, 12, true));
                $itemBancoHoras->setNrvinculom($nrvinculom);
                $itemBancoHoras->setNrbancohoras($parametro['NRBANCOHORAS']);
                $itemBancoHoras->setDtinicioaltebhoras($dtadmissao);
                $itemBancoHoras->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $itemBancoHoras->setCdoperultatu($this->hcmParameters->getCdoperador());
                $itemBancoHoras->setDtinclusao(DateUtil::getDataAtual(false));
                $itemBancoHoras->setDtultatu(DateUtil::getDataAtual(false));
                $itemBancoHoras->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                $itemBancoHoras->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                
                $this->entityManager->persist($itemBancoHoras);
                $this->entityManager->flush($itemBancoHoras);
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Relaciona a os perfis definidos como parametros para um parceiro de negocio
     * @var string $nrparcnegocio
     * @return void
     */
    private function setParametrosAdmissaoPerfisAcesso(string $nrparcnegocio): void {
        foreach($this->parametrosAdmissao as $parametro){
            $arrayNrperfilacesso = !is_null($parametro['NRPERFILACESSO']) ? explode(',', $parametro['NRPERFILACESSO']) : [];

            foreach($arrayNrperfilacesso as $nrperfilacesso) {
                $data = $this->setDataPerfacessparcOperador($nrperfilacesso, $nrparcnegocio);
                $this->importacaoPlanilhaVinculoRepository->novoPerfacessparcAdmissao($data);
            }
        }
    }

    private function setDataPerfacessparcOperador($nrperfilacesso, $nrparcnegocio){
        return $row = [
            'NRPERFILACESSO'  => $nrperfilacesso,
            'NRPARCNEGOCIO'   => $nrparcnegocio,
            'NRORG'           => $this->hcmParameters->getNrorgTrabalho(),
            'IDATIVO'         => 'S',
            'DTINCLUSAO'      => DateUtil::getDataAtual(false),
            'DTULTATU'        => DateUtil::getDataAtual(false),
            'CDOPERINCLUSAO'  => $this->hcmParameters->getCdoperador(),
            'CDOPERULTATU'    => $this->hcmParameters->getCdoperador()];
    }

    private function novoRelacionaparc($nrorg, $nrtiporelaciona, $nrparcnegocio, $dtfimrelacionamento, $nrparcnegrelac)
    {
        $relacionaparc = new $this->relacionaparc;

        $relacionaparc->setNrorg($nrorg);
        $relacionaparc->setDtinclusao(DateUtil::getDataAtual());
        $relacionaparc->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $relacionaparc->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $relacionaparc->setDtultatu(DateUtil::getDataAtual());
        $relacionaparc->setCdoperultatu($this->hcmParameters->getCdoperador());
        $relacionaparc->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $relacionaparc->setNrparcnegocio($nrparcnegocio);
        $relacionaparc->setNrtiporelaciona($nrtiporelaciona);
        $relacionaparc->setNrparcnegrelac($nrparcnegrelac);
        //$relacionaparc->setDtfimrelacionamento($dtfimrelacionamento);
        $relacionaparc->setIdativo('S');
        $relacionaparc->setNrrelacionaparc(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'RELACIONAPARC', 1, 10, true));

        $this->entityManager->persist($relacionaparc);
        $this->entityManager->flush($relacionaparc);

        return $relacionaparc;
    }

    private function calculaDtfimcontrdetermin($dtfimcontrato, $nrdias, $nrdiasprorr, $dtadmissaovinc) { 
        if($dtfimcontrato) {
            return $dtfimcontrato;
        } else if((!is_nan($nrdias) && $nrdias > 0) || (!is_nan($nrdiasprorr) && $nrdiasprorr > 0)) {
            if(!$nrdias)
                $nrdias = 0;
            if(!$nrdiasprorr)
                $nrdiasprorr = 0;

            $somaDias = $nrdias + $nrdiasprorr - 1;

            return $dtfimcontrato = DateUtil::adicionaIntervalo($dtadmissaovinc, $somaDias);
        } else {
            return $dtfimcontrato = null;
        }
    }

}