<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S1000Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    public function getItensParaEnvioS1000($params) {
        try {
            $sql = <<<SQL
SELECT EH.NRESTRUTURAM as NRCHAVE,
   '' as NRTABELA,
   DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
   NVL(EM.DTESOCIAL,EH.DTMESCOMPETENC) as DTMESCOMPETENC
FROM ESTRUTURAM EM
JOIN ESTRUTURAH EH ON EH.NRORG = EM.NRORG
                AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                          WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                            AND ESTRUTURAH.NRESTRUTURAM = EH.NRESTRUTURAM
                                            AND ESTRUTURAH.NRORG = EH.NRORG)
/*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
LEFT JOIN (SELECT EC.NRORG, EH.NRESTRUTURAM AS NRCHAVE, COUNT(*) AS ENVIOS
           FROM ESO_ENVIOCADASTROS EC
           JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                             AND EV.NMEVENTO = 'S-1000'
           JOIN ESTRUTURAH EH ON EH.NRORG = EC.NRORG
                             AND EH.NRESTRUTURAM = EC.NRCHAVE
          WHERE EC.NRORG = :P_NRORG
            AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
            AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
            AND EC.IDOPERACAO <> 'E' /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
            AND EC.NRENVIOEXCL IS NULL /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
            AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
          GROUP BY EC.NRORG, EH.NRESTRUTURAM
          UNION ALL
          SELECT EC.NRORG, EH.NRESTRUTURAM AS NRCHAVE, COUNT(*) AS ENVIOS
           FROM ESO_CADASTRO EC
           JOIN ESTRUTURAH EH ON EH.NRORG = EC.NRORG
                             AND EH.NRESTRUTURAM = EC.NRCHAVE
          WHERE EC.NRORG = :P_NRORG
            AND EC.NMEVENTO = 'S-1000'
            AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
            AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
            AND EC.IDOPERACAO <> 'E' /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
            AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
          GROUP BY EC.NRORG, EH.NRESTRUTURAM) ENV ON ENV.NRCHAVE = EH.NRESTRUTURAM
WHERE EM.NRORG = :P_NRORG
AND EM.NRTIPOESTRUTURA = :P_NRTIPOESTRUTLEGAL
AND EM.NRESTRUTURAM = :P_NRESTRUTURAM
AND EM.DTINIVIGENCIA <= :P_DTMESCOMPETENC
AND (EM.DTFIMVIGENCIA >= :P_DTMESCOMPETENC OR EM.DTFIMVIGENCIA IS NULL)
AND SUBSTR(REPLACE(REPLACE(REPLACE(EH.CDCNPJESTRUT,'.',''),'/',''),'-',''),9,4) = '0001'
SQL;
            $params = array(
                "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                "P_NRTIPOESTRUTLEGAL" => $this->hcmParameters->getTipoEstruturaLegalOrg(),
                "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
                "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            );
            return $this->connection->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getDadosS1000($params) {
        try {
            $dataHistorico = \HCM\Util\DateUtil::getDataDeString($params['DTMESCOMPETENC'], \HCM\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA, true)->format("d/m/Y");
            
            $dados = $this->getDadosPrincipais($params['NRCHAVE'], $params['NRTPAMBIENTE'], $dataHistorico);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getDadosPrincipais($nrchave, $nrtpambiente, $dtmescompetenc) {
        $sql = <<<SQL
SELECT EM.NRESTRUTURAM CHAVE,
       --ideEmpregador
       DECODE(PN.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') tpInsc,
       CASE WHEN PN.CDTIPOINSCRICAO = 'CPF' OR EH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PN.NRINSCRICAOPARC ELSE SUBSTR(PN.NRINSCRICAOPARC,1,8) END nrInsc,
       --idePeriodo
       TO_CHAR(GREATEST(EM.DTINIVIGENCIA,TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EM.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR))),'YYYY-MM') iniValid,
       TO_CHAR(EM.DTFIMVIGENCIA, 'YYYY-MM') fimValid,
       --infoCadastro
       CT.CDESOCIAL classTrib,
       CASE WHEN PN.CDTIPOINSCRICAO = 'CPF'THEN ''ELSE '0' END indCoop,
       CASE WHEN PN.CDTIPOINSCRICAO = 'CPF'THEN ''ELSE '0' END indConstr,
       CASE WHEN EH.VRALIQDESON > 0 THEN 1 ELSE 0 END indDesFolha,
       '' indOpcCP,
       CASE WHEN CT.CDESOCIAL IN ('21','22') THEN '' ELSE DECODE(EH.IDTIPOEMPR,1,'S',2,'S','') END indPorte,
       '0' indOptRegEletron,
       '' cnpjEFR,
       --dadosIsencao
       NULL dadosIsencao,
       '' ideMinLei,
       '' nrCertif,
       '' dtEmisCertif,
       '' dtVerncCertif,
       '' nrProtRenov,
       '' dtProtRenov,
       '' dtDou,
       '' pagDou,
       --infoOrgInternacional
       NULL infoOrgInternacional,
       '' indAcordoIsenMulta,
       '' novaValidade
       --TO_CHAR(EM.DTINIVIGENCIA, 'YYYY-MM') novaValidade_iniValid,
       --TO_CHAR(EM.DTFIMVIGENCIA, 'YYYY-MM') novaValidade_fimValid
  FROM ESTRUTURAM EM
  JOIN ESTRUTURAH EH ON EH.NRORG = EM.NRORG
                    AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND EH.DTMESCOMPETENC = (SELECT * FROM(
                                               SELECT H1.DTMESCOMPETENC FROM ESTRUTURAH H1
                                              WHERE H1.NRORG = EH.NRORG
                                                AND H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
  JOIN PARCNEGOCIO PN ON PN.NRPARCNEGOCIO = EH.NRPARCNEGOCIO
                     AND PN.NRORG = EH.NRORG
  LEFT JOIN CLASSTRIBUT CT ON CT.NRCLASSTRIBUT = EH.NRCLASSTRIBUT
 WHERE EM.NRORG = :P_NRORG
   AND EM.NRESTRUTURAM = :P_NRCHAVE
   AND EM.NRTIPOESTRUTURA = :P_NRTIPOESTRUTLEGAL
SQL;

        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRTIPOESTRUTLEGAL" => $this->hcmParameters->getTipoEstruturaLegalOrg(),
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_INIVALIDADEAMBPR" => $this->hcmParameters->getEsoIniValidCadIniProdRest(),
            "P_INIVALIDADEAMBP" => $this->hcmParameters->getEsoIniValidCadIniProducao(),
            "P_INIVALIDADEAMBNS" => $this->hcmParameters->getEsoIniValidCadIniNacioSimples(),
            "P_NRCHAVE" => $nrchave,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
}