<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S2501Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function getItensParaEnvioS2501($params) {
        try {
            $sql = <<<SQL
/*getItensParaEnvioS2501*/
SELECT PR.NRPROCESSTRAB as NRCHAVE,
       '1' as NRTABELA,
       'I' as IDOPERACAO,
       PR.DTPERPGTO AS DTMESCOMPETENC
  FROM GPE_PROCESSTRAB PR
  JOIN GPE_INFOPROCTRAB IP ON IP.NRORG = PR.NRORG
                          AND IP.NRPROCESSTRAB = PR.NRPROCESSTRAB
 WHERE PR.NRORG = :P_NRORG
   AND PR.NRESTRUTURAM = :P_NRESTRUTURAM
   AND PR.DTPERPGTO = :P_DTMESCOMPETENC
 ORDER BY 1
SQL;
            $params = array(
                "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            );
            return $this->connection->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosS2501($params) {
        try {
            $dados = $this->getDadosPrincipais($params['NRENVIO'], $params['NRCHAVE'],  $params['NRESTRUTURAM']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getDadosPrincipais($nrenvio, $nrchave, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2501*/
SELECT PR.CDPROCESSTRAB CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) id,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') tpInsc,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END nrInsc,
       --ideProc
       REPLACE(REPLACE(PR.CDPROCESSTRAB,'.',''),'-','') nrProcTrab,
       TO_CHAR(PR.DTPERPGTO, 'YYYY-MM') perApurPgto,
       SUBSTR(PR.DSOBSERVACAO,0,999) obs,
       PR.NRPROCESSTRAB
  FROM GPE_PROCESSTRAB PR
  /*Empregador*/
  JOIN ESTRUTURAM EMP ON EMP.NRORG = :P_NRORG
                     AND EMP.NRESTRUTURAM = :P_NRESTRUTURAM
  JOIN ESTRUTURAH EMH ON EMH.NRORG = EMP.NRORG
                     AND EMH.NRESTRUTURAM = EMP.NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND NRESTRUTURAM = EMH.NRESTRUTURAM
                                                  AND NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE PR.NRORG = :P_NRORG
   AND PR.NRPROCESSTRAB = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam,
        );
        return $this->connection->fetchAssoc($sql, $params);
    
    }
    
    public function getDadosS2501ideTrab($params){
        try{
            $dados = $this->getDadosideTrab($params['NRPROCESSTRAB']);
            return $dados;
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getDadosideTrab($nrprocesstrab) {
        $sql = <<<SQL
/*getDadosS2501ideTrab*/
SELECT REPLACE(REPLACE(PH.NRCPFPESSOA,'.',''),'-','') cpfTrab,
       VM.NRPESSOA
  FROM GPE_INFOPROCTRAB IP
  JOIN GPE_VINCULOM VM ON VM.NRORG = IP.NRORG
                      AND VM.NRVINCULOM = IP.NRVINCULOM
  JOIN GPE_PESSOAH PH ON PH.NRORG = VM.NRORG
                     AND PH.NRPESSOA = VM.NRPESSOA
                     AND PH.DTMESCOMPETENC = (SELECT * FROM(SELECT H1.DTMESCOMPETENC FROM GPE_PESSOAH H1
                                               WHERE H1.NRPESSOA = PH.NRPESSOA
                                                 AND H1.NRORG = PH.NRORG
                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 ORDER BY H1.DTMESCOMPETENC DESC)WHERE ROWNUM = 1)
 WHERE IP.NRORG = :P_NRORG
   AND IP.NRPROCESSTRAB = :P_NRPROCESSTRAB
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRPROCESSTRAB" => $nrprocesstrab,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2501calcTrib($params){
        try{
            $dados = $this->getDadoscalcTrib($params['NRPESSOA']);
            return $dados;
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getDadoscalcTrib($nrpessoa) {
        $sql = <<<SQL
/*getDadosS2501calcTrib*/
SELECT TO_CHAR(CF.DTOCORRENCIA, 'YYYY-MM') perRef,
       SUM(DECODE(IC.NREVENTOM,4500,IC.VREVENTOFOLHA,0)) vrBcCpMensal,
       SUM(DECODE(IC.NREVENTOM,4505,IC.VREVENTOFOLHA,0)) vrBcCp13,
       0 vrRendIRRF,
       0 vrRendIRRF13
  FROM GPE_INFOPROCTRAB IP
  JOIN GPE_VINCULOM VM ON VM.NRORG = IP.NRORG
                      AND VM.NRVINCULOM = IP.NRVINCULOM
  JOIN FPA_ITECALCFOLHA IC ON IC.NRORG = IP.NRORG
                          AND IC.NRVINCULOM = IP.NRVINCULOM
                          AND IC.NREVENTOM IN (4500,4505,4510,4515,4520,4525)
  JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = IC.NRORG
                          AND CF.NRCALCULOFOLHA = IC.NRCALCULOFOLHA
                          AND CF.NRTIPOMOVIMENT = 100 /*PROCESSO TRABALHISTA*/
                          AND CF.NRTPMODALIDCAL = 1
                          AND CF.DTOCORRENCIA BETWEEN IP.DTMESCOMPINI AND IP.DTMESCOMPFIM
 WHERE IP.NRORG = :P_NRORG
   AND VM.NRPESSOA = :P_NRPESSOA
 GROUP BY TO_CHAR(CF.DTOCORRENCIA, 'YYYY-MM')
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_NRPESSOA" => $nrpessoa,
        );
        return $this->connection->fetchAll($sql, $params);
    }
}
