<?php

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\Repositories;
use HCM\Geral\Repository\RemuneracaoOutrasFontesRepository;
use HCM\Util\HCMParameters;
use HCM\FPA\Services\ExportacaoExcelService;
use Zeedhi\Framework\HTTP\Kernel;
use HCM\Util\FPA;
use HCM\Util\DateUtil;
use Folha\Entities\EsoRemuneracaooutrasfontes;


class RemuneracaoOutrasFontesService {

    public function __construct(EntityManager $entityManager, RemuneracaoOutrasFontesRepository $remuneracaoOutrasFontesRepository, HCMParameters $hcmParameters, ExportacaoExcelService $exportacaoExcelService, Kernel $httpKernel)
    {
        $this->entityManager                     = $entityManager;
        $this->remuneracaoOutrasFontesRepository = $remuneracaoOutrasFontesRepository;
        $this->hcmParameters                     = $hcmParameters;
        $this->exportacaoExcelService            = $exportacaoExcelService;
        $this->httpKernel                        = $httpKernel;
        $this->EsoRemunOutrEmpr                  = Repositories::ESO_REMUNOUTREMPR;
    }
    
    public function existeRemuneracao($row)
    {   
        $existeRemuneracao = $this->remuneracaoOutrasFontesRepository->existeRemuneracao($row);
        return  $existeRemuneracao;
    }

    public function baixarPlanilhaRemuneracaoOutrasFontes()
    {
        $cabecalho = [
            "Nr. Pessoa",
            "CPF",
            "Cod Categ. Trab. eSocial",
            "Inscrição Fonte Pagadora",
            "Remuneração",
            "Tipo de Movimento"
        ];

        $dadosExemplo[0] = [
            "",
            "11111111111",
            "",
            "11111111111100",
            "5000",
            "1",
        ];

        $dadosExemplo[1] = [
            "234561",
            "",
            "37",
            "11111111111100",
            "5000",
            "4",
        ];
        
        return $this->exportacaoExcelService->gravarTemplate_1($this->hcmParameters->getNrorgTrabalho(), $cabecalho, 'Remuneracao_Outras_Fontes', $dadosExemplo);
    }
    
    
    public function importarPlanilhaRemuneracaoOutrasFontes($row)
    {
        $nrorg       = $this->hcmParameters->getNrorgTrabalho();
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $messages    = array(); 
        $dados       = $this->getDadosRemuneracaoOutrasFontesPlanilha($row['DSPATH'][0]);

        if(!empty($dados)) {
            foreach($dados as $lin => &$dado) {
                $res = $this->insereRemuneracaoOutrasFontesPlanilha($nrorg, $nrorgPadrao, $dado, $lin + 2);
               
                if(!empty($res)) {
                    foreach($res as $msg)
                    $messages[] = $msg;
                }    
            }
        }

        if(empty($dados)) {
            $messages[] =  array("msg" => "Não foi possível ler os dados da planilha. Por favor, gere um novo modelo e tente novamente.", "row" => 0, "field" => "N/A");
            
        }

        return $messages;
    }

    private function getDadosRemuneracaoOutrasFontesPlanilha($path)
    {
        $head = array(
                "NRPESSOA",
                "NRCPFPESSOA",
                "CDCODCATEG",
                "CDINSCOUTREMPR",
                "VRREMUNOUTREMPR",
                "NRTIPOMOVIMENT",
            );
            
        $file = $this->httpKernel->factoryFromFileData($path, FPA::getTempDirectory()."/");
        $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();
        
        $dateColumns = [];
        
        $file = \PHPExcel_IOFactory::load($caminhoNovo);
        $file->setActiveSheetIndex(0);
            
        return $this->exportacaoExcelService->preparaInfo_generic($file, $head, array('NRPESSOA', 'NRCPFPESSOA'), $dateColumns);
    }
    
    private function insereRemuneracaoOutrasFontesPlanilha($nrorg, $nrorgPadrao, $dado, $linha)
    {   
        try {
            $mensagens          =   [];
            $nrPessoaPlanilha   =   $dado['NRPESSOA'       ]  ?? null;
            $nrCpfPessoa        =   $dado['NRCPFPESSOA'    ]  ?? null;
            $cdCodCateg         =   $dado['CDCODCATEG'     ]  ?? null;
            $cdInscOutrEmpr     =   $dado['CDINSCOUTREMPR' ]  ?? null;
            $vrRemunOutrEmpr    =   $dado['VRREMUNOUTREMPR']  ?? null;
            $nrTipoMoviment     =   $dado['NRTIPOMOVIMENT' ]  ?? null;
            $dtMesCompetenc     =   $this->hcmParameters->getDtCompetencia();
            $cdEsocial          =   null;


            $validacaoPessoa = $this->validaDadosPlanilhaPessoa($nrPessoaPlanilha, $nrCpfPessoa, $dtMesCompetenc, $linha, $nrorg);
            $mensagens       = array_merge($mensagens, $validacaoPessoa['mensagens']);
            $nrPessoa        = $validacaoPessoa['nrPessoa'];


            if ($this->numeroValido($cdCodCateg)) {
                $cdEsocial = $this->entityManager->getRepository(Repositories::GPE_CATEGTRABESOC)->findOneBy(array('cdesocial' => $cdCodCateg));

                if(is_null($cdEsocial)) {
                    $mensagens[] = array(
                        "msg" => "O Código eSocial Categ. Trab. eSocial fornecido não foi encontrado. Por favor, insira um código válido. Consulte a tela 'Categoria do Trabalhador' para obter o código eSocial correto.",
                        "row" => $linha,
                        "field" => "Cod Categ. Trab. eSocial"
                    );
                }
            }
    
            if ($this->numeroValido($cdInscOutrEmpr)) {
                if (strlen($cdInscOutrEmpr) !== 14) {
                    $mensagens[] = array(
                        "msg" => "É necessário o preenchimento do campo 'Inscrição Fonte Pagadora' com um valor numérico até 14 dígitos. ",
                        "row" => $linha,
                        "field" => "Inscrição Fonte Pagadora"
                    );
                }
            } else {
                $mensagens[] = array(
                    "msg" => "É necessário o preenchimento do campo 'Inscrição Fonte Pagadora' com um valor numérico até 14 dígitos.",
                    "row" => $linha,
                    "field" => "Inscrição Fonte Pagadora"
                );
            }
    
            if ($this->numeroValido($vrRemunOutrEmpr)) {
                $vrRemunOutrEmpr = (float)$vrRemunOutrEmpr;
            } else {
                $mensagens[] = array(
                    "msg" => "É necessário o preenchimento do campo 'Remuneração' com um valor numérico.",
                    "row" => $linha,
                    "field" => "Remuneração"
                );
            }
                
            if ($this->numeroValido($nrTipoMoviment)) {
                $tipomoviment = $this->entityManager->getRepository(Repositories::FPA_TIPOMOVIMENT)->findOneBy(array("nrtipomoviment" => $nrTipoMoviment));

                if(is_null($tipomoviment)) {
                    $mensagens[] = array(
                        "msg" => "Nenhum Tipo de Movimento foi encontrado. Por favor, insira um número válido. ( (1) - MENSAL || (4) - 13 SALARIO )",
                        "row" => $linha,
                        "field" => "Cod Categ. Trab. eSocial"
                    );
                }
            } else {
                $mensagens[] = array(
                    "msg" => "É necessário o preenchimento do campo 'Tipo de Movimento' com um valor numérico.",
                    "row" => $linha,
                    "field" => "Tipo de Movimento"
                );
            }
            
            
            if(empty($mensagens)) {
                try { 
                    $row = [
                        'NRORG' => $nrorg,
                        'DTMESCOMPETENC' => $dtMesCompetenc,
                        'NRPESSOA' => $nrPessoa,
                        'CDINSCOUTREMPR' => $cdInscOutrEmpr,
                        'NRTIPOMOVIMENT' => $nrTipoMoviment
                    ];

                    $buscaRemuneracao = $this->remuneracaoOutrasFontesRepository->existeRemuneracao($row);
                    if(!empty($buscaRemuneracao)) {
                        $mensagens[] = array("msg" => "Já existe um registro de remuneração para esta pessoa, com a mesma competência, inscrição da fonte pagadora e tipo de movimento", "row" => $linha, "field" => "Remuneração duplicada");

                        return $mensagens;
                    }

                    $this->setImportacaoRemuneracaoOutrasFontes($nrorg, $nrPessoa, $cdCodCateg, $cdInscOutrEmpr, $vrRemunOutrEmpr, $dtMesCompetenc, $nrTipoMoviment);
            
                    return;
                } catch (\Exception $e) {
                    throw $e;
                }
            }

            return $mensagens;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function validaDadosPlanilhaPessoa($nrPessoaPlanilha, $nrCpfPessoa, $dtMesCompetenc, $linha, $nrorg)
    {   
        try {
            $mensagens = [];
            $pessoa    = null;
            $nrPessoa  = null;


            if ($this->numeroValido($nrPessoaPlanilha)) {
                $pessoa = $this->getPessoaPlanilha($nrorg, $nrPessoaPlanilha, $dtMesCompetenc);
            } else if ($this->numeroValido($nrCpfPessoa)) {
                $pessoa = $this->getPessoaCpf($nrorg, $dtMesCompetenc, $nrCpfPessoa);
            }
            
            if (empty($pessoa) ) {
                $mensagens[] = array(
                    "msg" => "O número da pessoa ou o CPF fornecido não foram encontrados. Por favor, certifique-se de que pelo menos um desses campos esteja corretamente preenchido.",
                    "row" => $linha,
                    "field" => "Nr. Pessoa/CPF"
                );
            } else {
                if (is_object($pessoa) && method_exists($pessoa, 'getNrpessoa')) {
                    $nrPessoa = $pessoa->getNrpessoa();
                } elseif (is_array($pessoa) && isset($pessoa[0]["NRPESSOA"])) {
                    $nrPessoa = $pessoa[0]["NRPESSOA"];
                }
            }

            return [
                'mensagens' => $mensagens,
                'nrPessoa' => $nrPessoa
            ];

        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function numeroValido($numero)
    {   
        return !is_null($numero) && is_numeric($numero);
    }

    private function getPessoaPlanilha($nrorg, $nrPessoaPlanilha, $dtMesCompetenc)
    {
        return $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaAtual(
            $nrorg,
            $nrPessoaPlanilha,
            $dtMesCompetenc
        );
    }   

    private function getPessoaCpf($nrorg, $dtMesCompetenc, $nrPessoaPlanilha)
    {
        return $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaCpf(
            $nrorg,
            $dtMesCompetenc,
            $nrPessoaPlanilha
        );
    }


    private function setImportacaoRemuneracaoOutrasFontes($nrorg, $nrPessoa, $cdCodCateg, $cdInscOutrEmpr, $vrRemunOutrEmpr, $dtMesCompetenc, $nrTipoMoviment)
    {
        try {
            $this->entityManager->beginTransaction();
            $dtMesCompetenc  = DateUtil::getDataDeString($dtMesCompetenc , DateUtil::FORMATO_BRASILEIRO, true);
            $envioRemuneracaoOutrasFontes = new $this->EsoRemunOutrEmpr;
            $envioRemuneracaoOutrasFontes->setNrorg($nrorg);
            $envioRemuneracaoOutrasFontes->setNrpessoa($nrPessoa);
            $envioRemuneracaoOutrasFontes->setCdcodcateg($cdCodCateg);
            $envioRemuneracaoOutrasFontes->setNrremunoutrempr(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'ESO_REMUNOUTREMPR', 1, 12, true));
            $envioRemuneracaoOutrasFontes->setCdinscoutrempr($cdInscOutrEmpr);
            $envioRemuneracaoOutrasFontes->setNrtipomoviment($nrTipoMoviment);
            $envioRemuneracaoOutrasFontes->setDtmescompetenc($dtMesCompetenc);
            $envioRemuneracaoOutrasFontes->setIdativo('S');
            $envioRemuneracaoOutrasFontes->setVrremunoutrempr($vrRemunOutrEmpr);
            
            $envioRemuneracaoOutrasFontes->setDtinclusao(DateUtil::getDataAtual());
            $envioRemuneracaoOutrasFontes->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $envioRemuneracaoOutrasFontes->setNrorginclusao($nrorg);
            $envioRemuneracaoOutrasFontes->setDtultatu(DateUtil::getDataAtual());
            $envioRemuneracaoOutrasFontes->setCdoperultatu($this->hcmParameters->getCdoperador());
            $envioRemuneracaoOutrasFontes->setNrorgultatu($nrorg);
            $this->entityManager->persist($envioRemuneracaoOutrasFontes);
            $this->entityManager->flush($envioRemuneracaoOutrasFontes);
            $this->entityManager->commit();
            
            return;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
}