<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Notification;
use HCM\PTO\Services\MotivoSituFuncService;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\Util\Util;
use Doctrine\ORM\EntityManager;

class MotivoSituFuncController extends Crud {
    
    protected $dataSourcePto_motivositufunc  = 'pto_motivositufunc';
    protected $ptoMotivoSituFunc_setMotivoSituFunc = 'ptoMotivoSituFunc_setMotivoSituFunc';
    protected $parameterBagColumns = array('P_NRORG', 'P_DTMESCOMPETENC', 'P_NRORG_PADRAO');
    
    public function __construct (Manager $dataSourceManager, EntityManager $entityManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, MotivoSituFuncService $motivoSituFuncService){
        $this->parameterBag = $parameterBag;
        $this->EntityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->motivoSituFuncService = $motivoSituFuncService;
        parent::__construct($dataSourceManager);
    }
    
    public function getMotivoSitufunc (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourcePto_motivositufunc);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
        
    }
    
    public function setMotivoSituFunc (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "ptoMotivoSituFunc_setMotivoSituFunc";
            foreach($request->getDataSet()->getRows() as $index => &$row) {
                if(!array_key_exists('NRSITUFUNCM', $row)){
                    $row['NRSITUFUNCM'] = null;
                }
                if(!array_key_exists('NRMOTIVOOCORFREQ', $row)){
                    $row['NRMOTIVOOCORFREQ'] = null;
                }
                if(!array_key_exists('NRMOTIVOAFASTA', $row)){
                    $row['NRMOTIVOAFASTA'] = null;
                }
                
                $valida = $this->motivoSituFuncService->verificaExistenciaMotivoSituFunc($row['NRORG'], $row['NRMOTIVOOCORFREQ'], $row['NRSITUFUNCM'], $row['NRMOTIVOSITUFUNC']);
                
                if (empty($valida)) {
                    if(array_key_exists('NRMOTIVOSITUFUNC', $row)){
                        if(empty($row['NRMOTIVOSITUFUNC'])){
                            $row['NRMOTIVOSITUFUNC'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_MOTIVOSITUFUNC', 1, 12, true);
                        }
                    } else {
                        $row['NRMOTIVOSITUFUNC'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_MOTIVOSITUFUNC', 1, 12, true);
                    }
                } else {
                    $response->addNotification(new Notification("Já existe um cadastro para esse mesmo Motivo de Ocorrência e/ou Situação Funcional", Notification::TYPE_ERROR));
                    return;
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteMotivoSitufunc(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = "ptoMotivoSituFunc_setMotivoSituFunc";
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }

     protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}