<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;

class PontosAcessoController extends Crud {

    protected $dataSourceGetPontosAcesso       = 'gpePontoAcesso_getPontosAcesso';
    protected $dataSourceGetPontosAcessoREP    = 'gpePontoAcesso_getPontosAcessoREP';
    protected $dataSourceGetTipoPontos       = 'gpePontoAcesso_getTipoPonto';
    protected $dataSourceSetPontosAcesso       = 'gpePontoAcesso_setPontosAcesso';
    protected $dataSourceGetColetorPontoAcesso = 'gpeColetor_getColetorPontoAcesso';
    protected $dataSourceSetColetorPontoAcesso = 'gpeColetor_setColetorPontoAcesso';


    protected $parameterBagColumns = array('NRORG', 'NRORGPADRAO', 'IDATIVO','NRESTRUTURAM', 'NRPONTOACESSO');
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag){
        $this->parameterBag = $parameterBag;
        parent::__construct($dataSourceManager);
    }

    public function getColetorPontosAcesso (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetColetorPontoAcesso);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function getPontosAcesso (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetPontosAcesso);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function getPontosAcessoREP (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetPontosAcessoREP);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function getTipoPontos (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetTipoPontos);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setColetorPontosAcesso (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetColetorPontoAcesso;
        try{
            foreach($request->getDataSet()->getRows() as $row) {

                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = $row['DTATUAL'];
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    $row['NRORG'] = $row['NRORGTRAB'];
                    $row['NRCOLETORPONTOACESSO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_COLETORPONTOACESSO', 1, 12, true);
                } else {
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setPontosAcesso (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetPontosAcesso;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                $row['NRTPREGIPONTO'] = isset($row['NRTPREGIPONTO'])?$row['NRTPREGIPONTO']:null;
                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = $row['DTATUAL'];
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    $row['NRORG'] = $row['NRORGTRAB'];
                    $row['NRPONTOACESSO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_PONTOACESSO', 1, 12, true);
                } else {
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteColetorPontosAcesso (DTO\Request\Row $row, DTO\Response $response) {
        try{
            $dataSet = new DataSet($this->dataSourceSetColetorPontoAcesso, array($row->getRow()));
            $deletedRows = $this->dataSourceManager->delete($dataSet);

            $response->addDataSet(new DataSet($this->dataSourceSetColetorPontoAcesso, array('type' => 'success', $deletedRows)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deletePontosAcesso (DTO\Request\Row $row, DTO\Response $response) {
        try{
            $dataSet = new DataSet($this->dataSourceSetPontosAcesso, array($row->getRow()));
            $deletedRows = $this->dataSourceManager->delete($dataSet);

            $response->addDataSet(new DataSet($this->dataSourceSetPontosAcesso, array('type' => 'success', $deletedRows)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}