<?php

namespace HCM\PTO\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;

class VinculoBancoHorasService {

    public function __construct (EntityManager $entityManager, HCMParameters $hcmParameters){
        $this->hcmParameters            = $hcmParameters;
        $this->entityManager            = $entityManager;
        $this->alteBancoHorasRepository = Repositories::PTO_ALTEBANCOHORAS;
    }

    public function analisaBancoHorasAtivo($row){
        $result = $this->entityManager->getRepository(Repositories::PTO_ALTEBANCOHORAS)->retornaAlteBancoHorasAtiva($this->hcmParameters->getNrorgTrabalho(), $row['NRVINCULOM'], $row['DTINICIOALTEBHORAS'], $row['DTFIMALTEBHORAS'], $row['NRALTEBANCOHORAS']);
        if($result == true){
            return $result;
        }
        return false;
    }
    
    /**
     *  Rotina responsável pelo cadastro automático de alterações de banco de horas
     * 
     *  @param  Array       $vinculos
     *  @param  string      $bancoHoras
     *  @param  string      $dtinicioaltebhoras
     *  @param  string      $dtfimaltebhoras
     * 
     *  @return boolean     success
     * 
     */
    public function cadastroAutomaticoAlterBancoHoras($vinculos, $bancoHoras, $dtinicioaltebhoras, $dtfimaltebhoras = null) {
        $nrorg      = $this->hcmParameters->getNrorgTrabalho();
        $cdoperador = $this->hcmParameters->getCdoperador();
        
        if (!is_array($vinculos)) {
            $vinculos = array($vinculos);
        }

        try {

            $this->entityManager->beginTransaction();

            foreach ($vinculos as $index => $vinculo) {
                $alteBancoHoras = $this->entityManager->getRepository(Repositories::PTO_ALTEBANCOHORAS)->findOneBy(array('nrorg' => $nrorg, 'nrbancohoras' => $bancoHoras, 'nrvinculom' => $vinculo));
                if (empty($alteBancoHoras)) {
                    $novoCadastroAlteBancoHoras = new $this->alteBancoHorasRepository;

                    $novoCadastroAlteBancoHoras->setNraltebancohoras(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'PTO_ALTEBANCOHORAS', 1, 22, true));
                    $novoCadastroAlteBancoHoras->setNrorg($nrorg);
                    $novoCadastroAlteBancoHoras->setNrbancohoras($bancoHoras);
                    $novoCadastroAlteBancoHoras->setNrvinculom($vinculo);
                    $novoCadastroAlteBancoHoras->setDtinicioaltebhoras(DateUtil::getDataDeString($dtinicioaltebhoras, DateUtil::FORMATO_BRASILEIRO, true));
                    $novoCadastroAlteBancoHoras->setDtfimaltebhoras(DateUtil::getDataDeString($dtfimaltebhoras, DateUtil::FORMATO_BRASILEIRO, true));
                    $novoCadastroAlteBancoHoras->setIdativo('S');
                    $novoCadastroAlteBancoHoras->setNrorginclusao($nrorg);
                    $novoCadastroAlteBancoHoras->setDtinclusao(new \DateTime());
                    $novoCadastroAlteBancoHoras->setCdoperinclusao($cdoperador);
                    $novoCadastroAlteBancoHoras->setNrorgultatu($nrorg);
                    $novoCadastroAlteBancoHoras->setDtultatu(new \DateTime());
                    $novoCadastroAlteBancoHoras->setCdoperultatu($cdoperador);

                    $this->entityManager->persist($novoCadastroAlteBancoHoras);
                }
            }

            $this->entityManager->flush();
            $this->entityManager->commit();
            return true;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }

    }
}