<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use Doctrine\ORM\EntityManager;
use HCM\SST\Services\MonitoracaoBiologicaService;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request\DataSet;

class MonitoracaoBiologicaController extends Crud {

    protected $parameterBagColumns = array('P_NRORG', 'P_NRESTRUTURAM');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager, MonitoracaoBiologicaService $monitoracaoBiologicaService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager; 
        $this->monitoracaoBiologicaService = $monitoracaoBiologicaService;
        parent::__construct($dataSourceManager);
    }

    public function getMonitoracaoBiologica(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "sst_respmonbio_get";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setMonitoracaoBiologica(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_respmonbio";
            foreach ($request->getDataSet()->getRows() as &$row) {
            
                $row['NRPARCREGAMB'] = $row['NRPARCREGAMB']??null;
                $row['DTINIRESMONBIO'] = $row['DTINIRESMONBIO']??null;
                $row['DTFIMRESMONBIO'] = $row['DTFIMRESMONBIO']??null;
                $row['NRESTRUTURAM'] = $row['NRESTRUTURAM']??null;
                $row['IDATIVO'] = $row['IDATIVO']?? 'S';

                $existeMonitoracaoBiologica = $this->monitoracaoBiologicaService->existeMesmoPeriodoMonitoracaoBiologica($row);
                if(!empty($existeMonitoracaoBiologica) && ($existeMonitoracaoBiologica['NRMONBIO'] != $row['NRMONBIO'] || !isset($row['NRMONBIO'])) ) {
                    $response->addNotification(new Notification("Já existe um cadastro de Monitoração Biológica para o período informado", Notification::TYPE_ALERT));
                    return;
                }
               
                if($row['__is_new'] === true){
                   $row['NRMONBIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_RESPMONBIO', 1, 12, true); 
                }

                parent::save($request, $response);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteMonitoracaoBiologica(DTO\Request\DataSet $request, DTO\Response $response)
    {   
        try {
            $this->dataSourceName = "sst_respmonbio";
            $this->dataSourceName = $this->dataSourceName;
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition)
    {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria)
    {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 