<?php

namespace HCM\SST\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use Folha\Entities\SstGherisco;
use HCM\SST\Repository\GheHomogenExposicaoRepository;


class GheHomogenExposicaoService {
    
    protected $entityManager;
    protected $hcmParameters;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, GheHomogenExposicaoRepository $gpeHomogenExposicaoRepository ) {
        $this->entityManager    = $entityManager;
        $this->hcmParameters    = $hcmParameters;
        $this->SstGherisco = Repositories::SST_GHERISCO;
        $this->gpeHomogenExposicaoRepository = $gpeHomogenExposicaoRepository;
    }

    public function setCadastroAutomaticoGHE($row)
    {   
        $row['DTINIVIGENCIA']     = $row['DTINIVIGENCIA'] ?? null;
        $row['DTFIMVIGENCIA']     = !empty($row['DTFIMVIGENCIA']) ? $row['DTFIMVIGENCIA'] : null;
        $row["NRSETOR"]           = $row["NRSETOR"] ?? null;
        $row["NRSITUGERADORA"]    = $row["NRSITUGERADORA"] ?? null;
        $row["NRCRITERIOEXPOSI"]  = $row["NRCRITERIOEXPOSI"] ?? null;
        $row["DSMEDIDASCTRADM"]   = $row["DSMEDIDASCTRADM"] ?? null;
        $row["DSMEDIDASCTREPI"]   = $row["DSMEDIDASCTREPI"] ?? null;
        $row["DTMEDICAO"]         = $row["DTMEDICAO"] ?? null;
        $row["DSMEDICAO"]         = $row["DSMEDICAO"] ?? null;
        $row["NRCLASSRISCO"]      = $row["NRCLASSRISCO"] ?? null;
        $row["NRFREQUENCIA"]      = $row["NRFREQUENCIA"] ?? null;
        $row["NRNIVELACAO"]       = $row["NRNIVELACAO"] ?? null;
        $row["NRCLASSEFEITO"]     = $row["NRCLASSEFEITO"] ?? null;
        $row["DSLOCAL"]           = $row["DSLOCAL"] ?? null;
        
        foreach ($row['NRESTRUTURAM'] as $estruturaKey => $estrutura) {
            foreach ($row['NROCUPACAOM'] as $ocupacaoKey => $ocupacao) {
                foreach ($row['NRFATORRISCO'] as $fatorRiscoKey => $fatorRisco) {
                    $existeGHE = $this->gpeHomogenExposicaoRepository->existeGHE($row, $estrutura, $ocupacao, $fatorRisco);
                    
                    if(!$existeGHE) {
                        $insereGHE = $this->setGHE($row, $estrutura, $ocupacao, $fatorRisco);
                    }
                }
            }
        }
        return;
    }

     private function setGHE($row, $estrutura, $ocupacao, $fatorRisco)
    {
        try {
            $this->entityManager->beginTransaction();

            $envioNovoGHE = new $this->SstGherisco;
            $envioNovoGHE->setNrorg($row["NRORG"]);
            $envioNovoGHE->setNrgherisco(\HCM\Util\NovoCodigo::geraCodigo($row["NRORG"], 'SST_GHERISCO', 1, 12, true));
            $envioNovoGHE->setNrfatorrisco($fatorRisco);
            $envioNovoGHE->setNrocupacaom($ocupacao);
            $envioNovoGHE->setNrestruturam($estrutura);
            $envioNovoGHE->setNrsitugeradora($row["NRSITUGERADORA"]);
            $envioNovoGHE->setNrsetor($row["NRSETOR"]);
            $envioNovoGHE->setDsmedidasctradm($row["DSMEDIDASCTRADM"]);
            $envioNovoGHE->setDsmedidasctrepi($row["DSMEDIDASCTREPI"]);
            $envioNovoGHE->setDtmedicao($row["DTMEDICAO"]);
            $envioNovoGHE->setDsmedicao($row["DSMEDICAO"]); 
            $envioNovoGHE->setNrclassrisco($row["NRCLASSRISCO"]);
            $envioNovoGHE->setNrfrequencia($row["NRFREQUENCIA"]);
            $envioNovoGHE->setNrnivelacao($row["NRNIVELACAO"]);
            $envioNovoGHE->setNrclassefeito($row["NRCLASSEFEITO"]);
            $envioNovoGHE->setNrcriterioexposi($row["NRCRITERIOEXPOSI"]);
            $envioNovoGHE->setDslocal($row["DSLOCAL"]);
            $envioNovoGHE->setIdgeraperic('N');
            $envioNovoGHE->setIdgerainsal('N'); 
            $envioNovoGHE->setDtfimvigencia(DateUtil::getDataDeString($row['DTFIMVIGENCIA'], DateUtil::FORMATO_BRASILEIRO, true));
            $envioNovoGHE->setDtinivigencia(DateUtil::getDataDeString($row['DTINIVIGENCIA'], DateUtil::FORMATO_BRASILEIRO, true));
            $envioNovoGHE->setIdativo('S');         
            $envioNovoGHE->setDtinclusao(DateUtil::getDataAtual());
            $envioNovoGHE->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $envioNovoGHE->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $envioNovoGHE->setDtultatu(DateUtil::getDataAtual());
            $envioNovoGHE->setCdoperultatu($this->hcmParameters->getCdoperador());
            $envioNovoGHE->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($envioNovoGHE);
            $this->entityManager->flush($envioNovoGHE);
            $this->entityManager->commit();
            return $envioNovoGHE;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
}